@@ -1802,6 +1802,101 @@ def any_hh_file(file_name):
       return p
   raise RuntimeError("Not an HHpred/HHalign/HHsearch file!")
 
+def any_a3m_file(file_name):
+  with open(file_name) as f:
+    data = f.read()
+  try:
+    a3m_info = read_a3m(data)
+  except KeyboardInterrupt : raise
+  except Exception as e :
+    pass
+  else :
+    return a3m_info
+  raise RuntimeError("Not an a3m file")
+
+def read_a3m(text):
+  """ Read text as a3m format
+      Ignore any lines starting with # at top
+      Lines starting with > or blank lines are separators and are ignored
+      Text between separators are sequence info.
+        First line is base sequence.  It cannot contain any lowercase or "-"
+        or "."
+        All other lines contain upper case characters (sequence matches),
+        lower case characters (insertions), and "-" or "." (gaps).
+      The total number of characters in base sequence must equal the
+        number of upper case characters plus number of gap characters, minus
+        the number of insertion characters in each other line.
+  """
+
+  # Get text as utf-8
+  if hasattr(text,'decode'):
+    text = text.decode(encoding='utf-8')
+
+  # Read in line representing each sequence
+  sequences = []
+  new_line = ""
+  for line in text.splitlines():
+    line = line.strip()
+    if not sequences and line.startswith("#"): continue # skip leading # lines
+    if not line or line.startswith(">"):  # separator
+      if new_line:
+        sequences.append(new_line)
+        new_line = ""
+      continue
+    new_line += line.strip().replace(" ","")
+  if new_line:
+    sequences.append(new_line)
+
+
+  # Check for illegal characters and length of lines
+  base_sequence = sequences[0]
+  n = len(base_sequence)
+  import re
+  for s in sequences:
+    if not ok_a3m_sequence(s, n = n, base_sequence = base_sequence):
+      return None
+
+  from libtbx import group_args
+  a3m_info = group_args(group_args_type = 'a3m_info',
+    base_sequence = base_sequence,
+    sequence_length = len(base_sequence),
+    sequences = sequences,
+     )
+  return a3m_info
+
+def ok_a3m_sequence(s, n = None, base_sequence = None):
+  """ Check a sequence and make sure it matches expectations for an a3m line
+  """
+  # Remove blanks/linefeeds and convert . to -
+  s = s.replace(" ","").replace("\r","").replace("\n","")
+  s = s.replace(".","-")
+
+  n_gap_chars = s.count("-")
+  s_all = s
+
+  # Get upper and lowercase
+  s = s.replace("-","")
+  # count lowercase/uppercase
+  n_upper = 0
+  n_lower = 0
+  n_other = 0
+  s_upper = ""
+  for c in s:
+    if c >="A" and c <= "Z":
+      n_upper += 1
+      s_upper += c
+    elif c >="a" and c <= "z":
+      n_lower += 1
+    else:
+      print(c)
+      n_other += 1
+
+  if n_other > 0:
+    return False
+  if n_upper + n_gap_chars == n:
+    return True
+  else:
+    return False
 def composition_from_sequence_file(file_name, log=None):
   if (log is None):
     log = sys.stdout
@@ -1911,14 +2006,20 @@ def get_number_of_dups(line):
   values = text.split()
   return max(1, len(values))
 
-def clear_empty_lines(text, apply_duplicate_multiple_chains = False):
+def clear_empty_lines(text, apply_duplicate_multiple_chains = False,
+    keep_labels = False):
   # First duplicate any multiple chains, then clear empty lines.
   if apply_duplicate_multiple_chains:
     text = duplicate_multiple_chains(text)
   # make empty lines just a blank line.  Includes >>> etc.
+  # If keep_labels, make the starting line for each group start with >
+
   new_lines=[]
   prev_line = ""
+  label_line = ""
   for line in text.splitlines():
+    if keep_labels and line.startswith(">"):
+      label_line = line
     if not line.replace(">","").replace(" ",""):
        line=""
     elif line.startswith(">"):
@@ -1926,16 +2027,21 @@ def clear_empty_lines(text, apply_duplicate_multiple_chains = False):
     line=line.replace("?","")
     if (not line) and (not prev_line):
       continue # skip blanks if dup or at beginning
+    if line and label_line:
+      new_lines.append(label_line)
+      label_line = ""
     new_lines.append(line)
     prev_line = line
   return "\n".join(new_lines)+"\n"
 
 def get_sequences(file_name=None,text=None,remove_duplicates=None,
      apply_duplicate_multiple_chains = False,
-     remove_unknowns = False):
+     remove_unknowns = False,
+     return_sequences_with_labels = False):
   # return simple list of sequences in this file. duplicates included
   #  unless remove_duplicates=True
   #  remove unknowns (X) if requested
+  #  If return_sequences_with_labels,  return sequence objects with labels
   if not text:
     if not file_name:
       raise Sorry("Missing file for get_sequences: %s" %(
@@ -1943,18 +2049,28 @@ def get_sequences(file_name=None,text=None,remove_duplicates=None,
     with open(file_name) as f:
       text = f.read()
   # clear any lines that have only > and nothing else
-  text=clear_empty_lines(text, apply_duplicate_multiple_chains)
-  chain_types=[]
+  text=clear_empty_lines(text, apply_duplicate_multiple_chains,
+    keep_labels = return_sequences_with_labels)
+
   ( sequences, unknowns ) = parse_sequence( text )
+
   simple_sequence_list=[]
+  sequence_object_list = []
   for sequence in sequences:
     if remove_duplicates and sequence.sequence in simple_sequence_list:
       continue # it is a duplicate
     elif remove_unknowns: # remove any X and take it
-      simple_sequence_list.append(sequence.sequence.replace("X",""))
-    else: # take it
+      sequence.sequence = sequence.sequence.upper().replace("X","")
       simple_sequence_list.append(sequence.sequence)
-  return simple_sequence_list
+      sequence_object_list.append(sequence)
+    else: # take it
+      sequence.sequence = sequence.sequence.upper()
+      simple_sequence_list.append(sequence.sequence)
+      sequence_object_list.append(sequence)
+  if return_sequences_with_labels:
+    return sequence_object_list
+  else:
+    return simple_sequence_list
 
 #####################################################################
 ####   Methods to try and guess chain types from sequences ##########
