@@ -128,6 +128,9 @@ class CCTBXParser(ParserBase):
     if program_class.program_name is None:
       program_class.program_name = self.prog
     self.prefix = self.prog.split('.')[-1]
+    # Windows dispatchers may have the .bat extension
+    if sys.platform == 'win32' and self.prefix.lower() == 'bat':
+      self.prefix = self.prog.split('.')[-2]
 
     # PHIL filenames
     self.data_filename = self.prefix + '_data.eff'
@@ -161,6 +164,7 @@ class CCTBXParser(ParserBase):
     self.data_manager = DataManager(
       datatypes=program_class.datatypes,
       custom_options=program_class.data_manager_options,
+      custom_master_phil_str=program_class.data_manager_custom_master_phil_str,
       logger=self.logger)
     self.namespace = None
 
@@ -319,6 +323,11 @@ class CCTBXParser(ParserBase):
     if self.namespace.show_defaults is not None:
       if self.namespace.attributes_level is None:
         self.namespace.attributes_level = 0
+      if self.program_class.show_data_manager_scope_by_default:
+        self.data_manager.master_phil.show(
+          expert_level=self.namespace.show_defaults,
+          attributes_level=self.namespace.attributes_level,
+          out=self.logger)
       self.master_phil.show(expert_level=self.namespace.show_defaults,
                             attributes_level=self.namespace.attributes_level,
                             out=self.logger)
@@ -335,6 +344,7 @@ class CCTBXParser(ParserBase):
     print('on %s by %s' % (time.asctime(), getpass.getuser()), file=self.logger)
     print('='*self.text_width, file=self.logger)
     print('', file=self.logger)
+    self.logger.flush()
 
     # process files
     if self.parse_files:
@@ -510,6 +520,8 @@ class CCTBXParser(ParserBase):
             except Sorry as e2:
               if e2.__str__().startswith('Unknown'):
                 self.unused_phil.append(phil)
+              else:
+                raise
           else:
             raise
         if processed_arg is not None:
@@ -517,6 +529,91 @@ class CCTBXParser(ParserBase):
         if data_manager_processed_arg is not None:
           data_manager_sources.append(data_manager_processed_arg)
 
+      # collect multiple DataManager label specifications from command-line
+      #   data_manager.miller_array.labels.name
+      #   data_manager.miller_array.user_selected_labels
+      def _get_last_object(phil_scope):
+        if phil_scope.is_definition:
+          return phil_scope
+        return _get_last_object(phil_scope.objects[0])
+
+      label_objects = []
+      type_objects = []
+      for source in data_manager_sources:
+        phil_object = _get_last_object(source)
+        if phil_object.full_path() in [
+          'data_manager.miller_array.labels.name',
+          'data_manager.miller_array.user_selected_labels',
+          'data_manager.map_coefficients.labels.name',
+          'data_manager.map_coefficients.user_selected_labels']:
+          label_objects.append(source)
+        if phil_object.full_path() == 'data_manager.model.type':
+          type_objects.append(source)
+
+      if len(label_objects) > 0:
+        print('  Found labels in command-line', file=self.logger)
+        print('  ----------------------------', file=self.logger)
+
+        # combine labels of the same datatype
+        datatypes = ['miller_array', 'map_coefficients']
+        for datatype in datatypes:
+          for parent_label_object in label_objects:
+            parent_object = _get_last_object(parent_label_object)  # name / user_selected_labels
+            pps = parent_object.primary_parent_scope  # miller_array
+            if parent_object.primary_parent_scope.name == 'datatype':
+              break
+          if parent_object.full_path() == 'data_manager.%s.labels.name' % datatype:
+            pps = parent_object.primary_parent_scope.primary_parent_scope
+          for child_label_object in label_objects:
+            if child_label_object is not parent_label_object \
+              and child_label_object in data_manager_data_sources \
+              and child_label_object in label_objects:
+              data_manager_sources.remove(child_label_object)
+              phil_object = _get_last_object(child_label_object)
+              if phil_object.full_path() == 'data_manager.%s.labels.name' % datatype:
+                phil_object = phil_object.primary_parent_scope
+              pps.objects.append(phil_object)
+              phil_object.primary_parent_scope = pps
+
+        # finally combine labels into one scope
+        label_phil = label_objects[0]
+        if len(label_objects) == len(datatypes):
+          for label_object in label_objects[1:]:
+            label_phil.adopt_scope(label_object)
+            if label_object in data_manager_sources:
+              data_manager_sources.remove(label_object)
+
+        print('', file=self.logger)
+
+        print('  Combined labels PHIL', file=self.logger)
+        print('  --------------------', file=self.logger)
+        tmp_working_phil = self.data_manager.master_phil.fetch_diff(label_phil)
+        print(tmp_working_phil.as_str(prefix='    '), file=self.logger)
+        print('', file=self.logger)
+
+      # set model types for each model
+      # if model type is specified, a model type needs to be specified
+      # for each model even if it is the default.
+      if len(type_objects) > 0:
+        model_names = self.data_manager.get_model_names()
+        if len(type_objects) != len(model_names):
+          raise Sorry('Please specify exactly one "model.type" for each model.')
+        print('  Matching model type PHIL:', file=self.logger)
+        print('  -------------------------', file=self.logger)
+        for model_name, type_object in zip(model_names, type_objects):
+          type_phil = self.data_manager.master_phil.fetch(source=type_object)
+          type_extract = type_phil.extract()
+          model_extract = type_extract.data_manager.model[0]
+          model_extract.file = model_name
+          print('    %s %s' % (model_extract.file, model_extract.type), file=self.logger)
+          new_type_object = self.data_manager.master_phil.format(python_object=type_extract)
+          for object in new_type_object.objects[0].objects:
+            if object.name == 'model':
+              e = object.extract()
+              if e.file == model_name:
+                type_object.objects[0].objects = [object]
+        print('', file=self.logger)
+
     if self.namespace.overwrite:  # override overwrite if True
       sources.append(iotbx.phil.parse('output.overwrite=True'))
 
@@ -547,14 +644,14 @@ class CCTBXParser(ParserBase):
     diff_phil = self.master_phil.fetch_diff(self.working_phil)
     paths = self.check_phil_for_paths(diff_phil)
     if len(paths) > 0:
-      files = []
-      dirs = []
+      files = set()
+      dirs = set()
       for path in paths:
         if path is not None:
           if os.path.isfile(path):
-            files.append(path)
+            files.add(path)
           elif os.path.isdir(path):
-            dirs.append(path)
+            dirs.add(path)
       if self.parse_files:
         self.process_files(files, message='Processing files from PHIL:')
       if self.parse_dir:
@@ -644,6 +741,9 @@ class CCTBXParser(ParserBase):
       or self.namespace.write_modified or self.namespace.write_all:
       print('Writing program PHIL file(s):', file=self.logger)
 
+    if self.data_manager.get_default_output_filename() is None:
+      self.data_manager.set_default_output_filename('cctbx_program')
+
     # write DataManager scope
     if self.namespace.write_data:
       if data_is_different:
@@ -740,7 +840,8 @@ or with pip run
 
 # =============================================================================
 def run_program(program_class=None, parser_class=CCTBXParser, custom_process_arguments=None,
-                unused_phil_raises_sorry=True, args=None, json=False, logger=None):
+                unused_phil_raises_sorry=True, args=None, json=False, logger=None,
+                hide_parsing_output=False):
   '''
   Function for running programs using CCTBXParser and the program template
 
@@ -760,6 +861,8 @@ def run_program(program_class=None, parser_class=CCTBXParser, custom_process_arg
     If True, get_results_as_JSON is called for the return value instead of get_results
   logger: multi_out
     For logging output (optional)
+  hide_parsing_output: bool
+    If True, hides the output from parsing the command-line arguments
 
   Returns
   -------
@@ -789,6 +892,12 @@ def run_program(program_class=None, parser_class=CCTBXParser, custom_process_arg
     logger.register('stdout', sys.stdout)
     logger.register('parser_log', StringIO())
 
+  program_logger = logger
+  if hide_parsing_output:
+    logger = multi_out()
+    logger.register('stderr', sys.stderr)
+    logger.register('parser_log', StringIO())
+
   # start timer
   t = show_times(out=logger)
 
@@ -807,7 +916,7 @@ def run_program(program_class=None, parser_class=CCTBXParser, custom_process_arg
   print('='*79, file=logger)
   task = program_class(parser.data_manager, parser.working_phil.extract(),
                        master_phil=parser.master_phil,
-                       logger=logger)
+                       logger=program_logger)
 
   # validate inputs
   task.validate()
