@@ -801,7 +801,11 @@ def show_exception_info_if_full_testing(prefix="EXCEPTION_INFO: "):
     if (is_done()): continue
     out.write(msg)
     flush = getattr(out, "flush", None)
-    if (flush is not None): flush()
+    if (flush is not None):
+      try:
+        flush()
+      except Exception as e:
+        pass # can happen if stale file handle
     done.append(out)
   return msg
 
@@ -1523,7 +1527,8 @@ class multi_out(object):
         old_label,
         new_label,
         new_file_object,
-        new_atexit_send_to=None):
+        new_atexit_send_to=None,
+        close_old_stream=True):
     """
     Replaces a registered stream with a new file. Dumps everything accumulated
     in stream to the file.
@@ -1537,11 +1542,14 @@ class multi_out(object):
     new_label : str
     new_file_object : file
     new_atexit_send_to : file, optional
+    close_old_stream :  bool, close existing stream
     """
     i = self.labels.index(old_label)
     old_file_object = self.file_objects[i]
-    new_file_object.write(old_file_object.getvalue())
-    old_file_object.close()
+    if hasattr(old_file_object, 'getvalue'):
+      new_file_object.write(old_file_object.getvalue())
+    if close_old_stream:
+      old_file_object.close()
     self.labels[i] = new_label
     self.file_objects[i] = new_file_object
     self.atexit_send_to[i] = new_atexit_send_to
@@ -1559,7 +1567,11 @@ class multi_out(object):
   def flush(self):
     for file_object in self.file_objects:
       flush = getattr(file_object, "flush", None)
-      if (flush is not None): flush()
+      if (flush is not None):
+        try:
+          flush()
+        except Exception as e:
+          pass # was closed
 
   def write(self, str):
     for file_object in self.file_objects:
@@ -2352,7 +2364,11 @@ def try_send_to_trash(path_name, delete_if_not_available=False,
         raise Sorry("This function not supported because the required module is "+
           "not installed.")
     else :
-      send2trash.send2trash(path_name)
+      try:
+        send2trash.send2trash(path_name)
+      except Exception as e:
+        print("Unable to send the directory %s to trash" %(path_name))
+        return 1 # indicate failure
 
 if sys.hexversion >= 0x03000000:
   unicode = str
@@ -2508,3 +2524,21 @@ git lfs install --local
 git lsf pull
 """%path)
   return not test
+
+class timer:
+  '''
+  Context manager for timing blocks of code
+  https://stackoverflow.com/questions/33987060/python-context-manager-that-measures-time
+
+  from libtbx.utils import timer
+
+  with timer():
+    <block of code to be timed>
+  '''
+  def __enter__(self):
+    self.time = time.perf_counter()
+    return self
+
+  def __exit__(self, type, value, traceback):
+    self.time = time.perf_counter() - self.time
+    print('Elapsed time (s): {}'.format(self.time))
