@@ -16,6 +16,8 @@ from collections import OrderedDict
 import mmtbx.f_model
 import sys
 from libtbx.test_utils import approx_equal
+from libtbx.utils import Sorry
+from cctbx import miller
 
 from mmtbx import masks
 from cctbx.masks import vdw_radii_from_xray_structure
@@ -69,13 +71,13 @@ class minimizer2(object):
     self.n = self.x.size()
     self.cntr=0
 
-  def run(self, use_curvatures=0):
+  def run(self, use_curvatures):
     self.minimizer = kbu_refinery.lbfgs_run(
       target_evaluator=self,
       min_iterations=self.min_iterations,
       max_iterations=self.max_iterations,
       use_curvatures=use_curvatures)
-    self(requests_f_and_g=True, requests_diag=False)
+    self(requests_f_and_g=True, requests_diag=use_curvatures)
     return self
 
   def __call__(self, requests_f_and_g, requests_diag):
@@ -241,6 +243,7 @@ class refinery(object):
   def __init__(self, fmodel, fv, alg, log = sys.stdout):
     assert alg in ["alg0", "alg2", "alg4", "alg4a"]
     self.log             = log
+    self.fv              = fv
     self.f_obs           = fmodel.f_obs()
     self.r_free_flags    = fmodel.r_free_flags()
     k_mask_overall       = fmodel.k_masks()[0]
@@ -248,7 +251,7 @@ class refinery(object):
     self.f_calc          = fmodel.f_calc()
     phase_source_init    = fmodel.f_model()
     k_total              = fmodel.k_total()
-    self.F               = [self.f_calc.deep_copy()] + fv.keys()
+    self.F               = [self.f_calc.deep_copy()] + list(self.fv.keys())
     n_zones_start        = len(self.F)
     r4_start             = fmodel.r_work4()
     r4_best              = r4_start
@@ -257,8 +260,18 @@ class refinery(object):
     r4_start             = None
     r4_best              = None
     self.fmodel_best     = None
+    self.n_regions       = None
     #
+
+    #alg_save = alg
+
     for it in range(5):
+
+      #if it in [0,1]:
+      #  alg = "alg4a"
+      #else:
+      #  alg = alg_save
+
       #
       if(it==1):
         r4_start         = self.fmodel.r_work4()
@@ -269,23 +282,20 @@ class refinery(object):
         if(abs(round(r4-r4_start,4))<1.e-4):
           break
         r4_start = r4
-      #if(it>0 and n_zones_start == len(self.F)): break
-      #
-      #if it>0:
-      #  self.F = [self.fmodel.f_model().deep_copy()] + self.F[1:]
-      self._print("cycle: %2d"%it)
-      self._print("  volumes: "+" ".join([str(fv[f]) for f in self.F[1:]]))
+
+      self._print("cycle: %2d (regions: %d)"%(it, len(self.F[1:])))
+      self._print("  Region #:                  "+"".join(["%7d"%fv[f] for f in self.F[1:]]))
+
       f_obs   = self.f_obs.deep_copy()
       if it>0: k_total = self.fmodel.k_total()
       i_obs   = f_obs.customized_copy(data = f_obs.data()*f_obs.data())
       K_MASKS = OrderedDict()
 
       if(alg.endswith('a')):
-        self.bin_selections = [self.f_calc.d_spacings().data()>3]
+        self.bin_selections = [self.f_calc.d_spacings().data()>4]
       else:
         self.bin_selections = thiken_bins(
           bins=bin_selections_input, n=50*len(self.F), ds=self.f_calc.d_spacings().data())
-
       for i_bin, sel in enumerate(self.bin_selections):
         d_max, d_min = f_obs.select(sel).d_max_min()
         #if d_max<3 or d_min<3: continue
@@ -339,7 +349,14 @@ class refinery(object):
           f_bulk_data_ += self.F[i_mask].data().select(sel)*k_mask
         f_bulk_data = f_bulk_data.set_selected(sel,f_bulk_data_ )
       #
-      self.update_F(K_MASKS)
+
+      #if it in [0,1]:
+      #  self.update_F(K_MASKS, cutoff=0)
+      #else:
+      #  self.update_F(K_MASKS, cutoff=0.03)
+      self.update_F(K_MASKS, cutoff=0.03)
+
+
       f_bulk = self.f_calc.customized_copy(data = f_bulk_data)
 
       if(len(self.F)==2):
@@ -379,17 +396,14 @@ class refinery(object):
         self.fmodel.update_all_scales(remove_outliers=False,
           apply_scale_k1_to_f_obs = APPLY_SCALE_K1_TO_FOBS)
         self._print(self.fmodel.r_factors(prefix="  "))
-
-      self.mc = self.fmodel.electron_density_map().map_coefficients(
-        map_type   = "mFobs-DFmodel",
-        isotropize = True,
-        exclude_free_r_reflections = False)
       #
       r4 = self.fmodel.r_work4()
       if(r4_best is not None and r4<=r4_best):
         r4_best = r4
         self.fmodel_best = self.fmodel.deep_copy()
+    #
     self.fmodel = self.fmodel_best
+    self.n_regions = len(self.F)-1
 
 
   #def update_k_masks(self, K_MASKS):
@@ -408,12 +422,14 @@ class refinery(object):
     if(self.log is not None):
       print(m, file=self.log)
 
-  def update_F(self, K_MASKS):
+  def update_F(self, K_MASKS, cutoff=0.03):
     tmp = []
     for i_mask, F in enumerate(self.F):
       k_masks = [k_masks_bin[1][i_mask] for k_masks_bin in K_MASKS.values()]
+      #print(i_mask, flex.mean(flex.double(k_masks)), [round(i,2) for i in k_masks])
       if(i_mask == 0):                        tmp.append(self.F[0])
-      elif moving_average(k_masks,2)[0]>=0.03: tmp.append(F)
+      #elif moving_average(k_masks,2)[0]>=cutoff: tmp.append(F)
+      elif flex.mean(flex.double(k_masks))>0.01: tmp.append(F)
       self.F = tmp[:]
 
   def _get_x_init(self, i_bin):
@@ -546,21 +562,201 @@ def filter_mask(mask_p1, volume_cutoff, crystal_symmetry,
     conn = conn / crystal_symmetry.space_group().order_z()
   return conn
 
+
+def modify(fofc, R, cutoff, crystal_gridding, mask=None):
+  G = fofc.g_function(R=R)
+
+  fft_map = fofc.fft_map(crystal_gridding = crystal_gridding)
+  fft_map.apply_volume_scaling()
+  map_data = fft_map.real_map_unpadded()
+
+  if mask is not None: map_data = map_data*mask
+
+  sel = map_data<cutoff
+  map_data = map_data.set_selected(sel, cutoff)
+
+  B = miller.structure_factor_box_from_map(
+    crystal_symmetry=fofc.crystal_symmetry(),
+    map=map_data, n_real=None,
+     anomalous_flag=False, include_000=True)
+  GB = B.g_function(R=R)
+  B = B.customized_copy(data = B.data()*GB)
+  fft_map = B.fft_map(crystal_gridding = crystal_gridding)
+  fft_map.apply_volume_scaling()
+  map_data = fft_map.real_map_unpadded()
+
+  print(flex.min(map_data), flex.max(map_data), flex.mean(map_data))
+
+  map_data = map_data.set_selected(map_data<abs(flex.min(map_data)), 0)
+
+  print(flex.min(map_data), flex.max(map_data), flex.mean(map_data))
+
+  F=B
+
+
+  #F = fofc.structure_factors_from_map(map = map_data, in_place_fft=False,
+  #  use_scale=True, use_sg=False)
+  #
+  #F = F.customized_copy(data = F.data()*G)
+  #
+  #fft_map = F.fft_map(crystal_gridding = crystal_gridding)
+  #fft_map.apply_volume_scaling()
+  #map_data = fft_map.real_map_unpadded()
+
+  mtz_dataset = F.as_mtz_dataset(column_root_label = "F")
+  mtz_object = mtz_dataset.mtz_object()
+  mtz_object.write(file_name = "map.mtz")
+
+  write_map_file(crystal_symmetry=fofc.crystal_symmetry(),
+    map_data=map_data, file_name="map.ccp4")
+
+  return map_data
+
+class mask_and_regions(object):
+  def __init__(self,
+      xray_structure,
+      crystal_gridding,
+      step=0.6,
+      r_sol=1.1,
+      r_shrink=0.9,
+      volume_cutoff=50,
+      wrapping=True,
+      force_symmetry=True,
+      log=None):
+    """
+    Split 0/1 traditional bulk-solvent mask into a series of isolated masks
+    covering the whole unit cell in P1.
+    """
+    self.crystal_gridding = crystal_gridding
+    self.crystal_symmetry = xray_structure.crystal_symmetry()
+    self.step = step
+    # Compute mask in P1
+    self.mask_p1 = mmtbx.masks.mask_from_xray_structure(
+      xray_structure           = xray_structure,
+      p1                       = True,
+      for_structure_factors    = True,
+      solvent_radius           = r_sol,
+      shrink_truncation_radius = r_shrink,
+      n_real                   = self.crystal_gridding.n_real(),
+      in_asu                   = False).mask_data
+    maptbx.unpad_in_place(map=self.mask_p1)
+    # Solvent fraction
+    self.solvent_content=100.*(self.mask_p1!=0).count(True)/self.mask_p1.size()
+    if(log is not None):
+      print("Solvent content: %7.3f"%self.solvent_content, file=log)
+    # Connectivity
+    co = maptbx.connectivity(
+      map_data                   = self.mask_p1,
+      threshold                  = 0.01,
+      preprocess_against_shallow = False, # XXX WHY False?
+      wrapping                   = wrapping)
+    if(force_symmetry and xray_structure.space_group().type().number() != 1):
+      co.merge_symmetry_related_regions(
+        space_group = xray_structure.space_group())
+    # Regions
+    self.conn = co.result().as_double() # 0 = protein, 1 = solvent, >1 = other
+    z = zip(co.regions(),range(0,co.regions().size()))
+    self.sorted_by_volume = sorted(z, key=lambda x: x[0], reverse=True)
+    self.n_regions_total = co.regions().size()
+    print("Total number of regions: %d"%self.n_regions_total, file=log)
+    #
+    if(log is not None):
+      print("Regions (V > %d):"%volume_cutoff, file=log)
+      print("  #   #   Vol. (A^3)    uc(%)", file=log)
+    self.small_selection = flex.size_t()
+    self.regions = []
+    cntr = 0
+    for i_seq, p in enumerate(self.sorted_by_volume):
+      v, i = p
+      # skip macromolecule
+      if(i==0): continue
+      # Region (i)selection
+      iselection = (self.conn==i).iselection()
+      # Skip small volume regions
+      volume = v*self.step**3
+      uc_fraction = v*100./self.conn.size()
+      if(volume_cutoff is not None and volume < volume_cutoff):
+        self.small_selection.extend(iselection)
+        continue
+      # Region
+      region = group_args(
+        i_seq       = cntr,
+        i           = i,
+        volume      = volume,
+        uc_fraction = uc_fraction,
+        iselection  = iselection)
+      cntr+=1
+      self.regions.append(region)
+      if(log is not None):
+        print("%3d %3d %12.3f %8.4f"%(region.i_seq, region.i, region.volume,
+          region.uc_fraction), file=log)
+    #
+    self.n_regions = len(self.regions)
+    print("Number of regions (V > %d): %d"%(
+      volume_cutoff, self.n_regions), file=log)
+
+  def get_selection(self, region):
+    result = flex.bool(self.conn.size(), region.iselection)
+    result.resize(self.conn.accessor())
+    return result
+
+  def get_region_asu_mask(self, region, map_data=None, write=False):
+    # This is ASU mask, NOT P1 !
+    selection = self.get_selection(region = region)
+    mask_i = flex.double(flex.grid(self.crystal_gridding.n_real()), 0)
+    mask_i = mask_i.set_selected(selection, 1)
+    if(map_data is not None):
+      mask_i = mask_i * map_data # may beed to divide by max value of map_data
+    if(write): # This write P1 mask for region
+      write_map_file(
+        crystal_symmetry = self.crystal_symmetry,
+        map_data         = mask_i,
+        file_name        = "mask_%s.mrc"%str(round(region.volume,3)))
+    return asu_map_ext.asymmetric_map(
+      self.crystal_symmetry.space_group().type(), mask_i).data()
+
+  def _mask_to_sf(self, mask_data, miller_array, d_min_cutoff=3):
+    # Cutoff incoming Miller array
+    result = miller_array.deep_copy()
+    d_spacings    = miller_array.d_spacings().data()
+    sel           = d_spacings >= 3
+    miller_array_ = miller_array.select(sel)
+    # Compute SF for truncated Miller array
+    mask_asu = asu_map_ext.asymmetric_map(
+      self.crystal_symmetry.space_group().type(), mask_data).data()
+    sf = miller_array_.structure_factors_from_asu_map(
+        asu_map_data = mask_asu, n_real = self.crystal_gridding.n_real())
+    # Inflate Miller array to original hkl set before returning
+    data = flex.complex_double(d_spacings.size(), 0)
+    data = data.set_selected(sel, sf.data())
+    return result.set().array(data=data)
+
+  def f_mask_whole(self, miller_array):
+    return self._mask_to_sf(mask_data=self.mask_p1, miller_array=miller_array)
+
+  def f_mask_whole_filtered(self, miller_array):
+    mask_p1_corrected = self.mask_p1.deep_copy()
+    sel = flex.bool(self.conn.size(), self.small_selection)
+    sel.resize(self.conn.accessor())
+    mask_p1_corrected = mask_p1_corrected.set_selected(sel, 0)
+    return self._mask_to_sf(
+      mask_data=mask_p1_corrected, miller_array=miller_array)
+
 class f_masks(object):
   def __init__(self,
+               mask_and_regions,
+               crystal_gridding,
                xray_structure,
-               step,
-               volume_cutoff=None,
-               mean_diff_map_threshold=None,
-               compute_whole=False,
-               largest_only=False,
-               wrapping=True, # should be False if working with ASU
-               f_obs=None,
-               r_sol=1.1,
-               r_shrink=0.9,
+               f_obs,
                f_calc=None,
+               r_free_flags=None,
+               mean_diff_map_threshold=0.5,
                log = None,
                write_masks=False):
+    """
+    Compute Fmask(whole), Fmask(main) and list of mosaic contributions.
+    Fmask match input miller array set but zeroed out in 3A-better resolution.
+    """
     adopt_init_args(self, locals())
     #
     self.d_spacings   = f_obs.d_spacings().data()
@@ -568,211 +764,253 @@ class f_masks(object):
     self.miller_array = f_obs.select(self.sel_gte3)
     #
     self.crystal_symmetry = self.xray_structure.crystal_symmetry()
-    # Compute mask in p1 (via ASU)
-    self.crystal_gridding = maptbx.crystal_gridding(
-      unit_cell        = xray_structure.unit_cell(),
-      space_group_info = xray_structure.space_group_info(),
-      symmetry_flags   = maptbx.use_space_group_symmetry,
-      step             = step)
     self.n_real = self.crystal_gridding.n_real()
-    # XXX Where do we want to deal with H and occ==0?
-    self._mask_p1 = self._compute_mask_in_p1()
-    self.solvent_content = 100.*(self._mask_p1 != 0).count(True)/\
-      self._mask_p1.size()
-    # Optionally compute Fmask from original whole mask, zero-ed at dmin<3A.
-    self.f_mask_whole = self._compute_f_mask_whole()
-    # Connectivity analysis
-    co = maptbx.connectivity(
-      map_data                   = self._mask_p1,
-      threshold                  = 0.01,
-      preprocess_against_shallow = False,
-      wrapping                   = wrapping)
-    if(xray_structure.space_group().type().number() != 1): # not P1
-      co.merge_symmetry_related_regions(
-        space_group = xray_structure.space_group())
     #
-    self.conn = co.result().as_double()
-    z = zip(co.regions(),range(0,co.regions().size()))
-    sorted_by_volume = sorted(z, key=lambda x: x[0], reverse=True)
-    #
-    f_mask_data_0   = flex.complex_double(f_obs.data().size(), 0)
-    self.f_mask_0   = None
-    self.FV         = OrderedDict()
-    self.mFoDFc_0   = None
-    diff_map        = None # mFo-DFc map computed using F_mask_0 (main mask)
-    self.regions    = OrderedDict()
-    small_selection = None
-    weak_selection  = None
+    f_mask_data_main = flex.complex_double(f_obs.data().size(), 0)
+    self.f_mask_main = None
+    self.FV          = OrderedDict()
+    self.mFoDFc_main = None
+    self.diff_map    = None # mFo-DFc map computed using F_mask_0 (main mask)
     #
     if(log is not None):
-      print("  #    volume_p1    uc(%) mFo-DFc: min,max,mean,sd", file=log)
+      print("Regions satisfying volume and map criteria:", file=log)
+      print("                                   mFo-DFc (sigma)", file=log)
+      print("  #   Vol. (A^3)    uc(%)     min     max    mean      sd", file=log)
     #
-    for i_seq, p in enumerate(sorted_by_volume):
-      v, i = p
-      self._region_i_selection = None # must be here inside the loop!
+    for region in self.mask_and_regions.regions:
       f_mask_i = None # must be here inside the loop!
-      # skip macromolecule
-      if(i==0): continue
-      # skip small volume and accumulate small volumes
-      volume = v*step**3
-      uc_fraction = v*100./self.conn.size()
-      if(volume_cutoff is not None and volume < volume_cutoff):
-        if(volume >= 10):
-          if(small_selection is None):
-            small_selection  = self._get_region_i_selection(i)
-          else:
-            small_selection |= self._get_region_i_selection(i)
-        continue
-      # Accumulate regions with volume greater than volume_cutoff (if
-      # volume_cutoff is defined). Weak density regions are included.
-      self.regions[i_seq] = group_args(
-        id          = i,
-        i_seq       = i_seq,
-        volume      = volume,
-        uc_fraction = uc_fraction)
       # Compute i-th region mask
-      mask_i_asu = self.compute_i_mask_asu(
-        selection = self._get_region_i_selection(i), volume = volume)
+      mask_i_asu = self.mask_and_regions.get_region_asu_mask(region = region)
       # Compute F_mask_0 (F_mask for main mask)
-      if(uc_fraction >= 1):
+      if(region.uc_fraction >= 1):
         f_mask_i = self.compute_f_mask_i(mask_i_asu)
-        f_mask_data_0 += f_mask_i.data()
-      elif(largest_only): break
+        f_mask_data_main += f_mask_i.data()
       # Compute mFo-DFc map using main mask (once done computing main mask!)
-      if(uc_fraction < 1 and diff_map is None):
-        diff_map = self.compute_diff_map(f_mask_data_0 = f_mask_data_0)
-      # Analyze mFo-DFc map in the i-th region
-      mi,ma,me,sd = None,None,None,None
-      if(diff_map is not None):
-        iselection = self._get_region_i_selection(i).iselection()
-        blob = diff_map.select(iselection)
-        mean_diff_map = flex.mean(diff_map.select(iselection))
-        mi,ma,me = flex.min(blob), flex.max(blob), flex.mean(blob)
-        sd = blob.sample_standard_deviation()
+      if(region.uc_fraction < 1 and self.diff_map is None and
+         self.f_calc is not None and mean_diff_map_threshold is not None):
+        self.diff_map = self.compute_diff_map(f_mask_data = f_mask_data_main)
+      if(self.diff_map is not None):
+        # Analyze mFo-DFc map in the i-th region
+        map_info = self._get_map_info(iselection = region.iselection)
+        # Skip regions with weak mean density
+        if(map_info.mean is not None and map_info.mean < mean_diff_map_threshold):
+          continue
         if(log is not None):
-          print("%3d"%i_seq,"%12.3f"%volume, "%8.4f"%round(uc_fraction,4),
-              "%7.3f %7.3f %7.3f %7.3f"%(mi,ma,me,sd), file=log)
-        # Accumulate regions with weak density into one region, then skip
-        if(mean_diff_map_threshold is not None):
-          if(mean_diff_map <= mean_diff_map_threshold):
-            if(mean_diff_map > 0.1):
-              if(weak_selection is None):
-                weak_selection  = self._get_region_i_selection(i)
-              else:
-                weak_selection |= self._get_region_i_selection(i)
-            continue
+          print("%3d"%region.i_seq,"%12.3f"%region.volume,
+                "%8.4f"%round(region.uc_fraction,4), map_info.string, file=log)
       else:
-        if(log is not None):
-          print("%3d"%i_seq,"%12.3f"%volume, "%8.4f"%round(uc_fraction,4),
-              "%7s"%str(None), file=log)
-      # Compute F_maks for i-th region
+        print("%3d"%region.i_seq,"%12.3f"%region.volume,
+              "%8.4f"%round(region.uc_fraction,4), file=log)
+      # Compute F_mask for i-th region
       if(f_mask_i is None):
         f_mask_i = self.compute_f_mask_i(mask_i_asu)
       # Compose result object
-      self.FV[f_mask_i] = [round(volume, 3), round(uc_fraction,1)]
+      self.FV[f_mask_i] = region.i_seq
+    # END OF LOOP OVER REGIONS
+
+
+
+    """
+
+    #self.FV          = OrderedDict()
+    #self.FV[f_obs.customized_copy(data = f_mask_data_main)] = 0
+
+    self.diff_map = self.compute_diff_map(f_mask_data = f_mask_data_main)
+
+    mask_ = mmtbx.masks.mask_from_xray_structure(
+      xray_structure           = self.xray_structure,
+      p1                       = True,
+      for_structure_factors    = True,
+      #solvent_radius           = 0.5,
+      #shrink_truncation_radius = 0.5,
+      n_real                   = self.n_real,
+      in_asu                   = False).mask_data
+    maptbx.unpad_in_place(map=mask_)
+
+    mc = self.mFoDFc_main
+    fft_map = mc.fft_map(
+      symmetry_flags   = maptbx.use_space_group_symmetry,
+      crystal_gridding = self.crystal_gridding)
+    fft_map.apply_volume_scaling()
+    map_data = fft_map.real_map_unpadded() #* mask_
+
+    map_data = map_data.set_selected(map_data<0,0)
+
+    #sgt = self.f_obs.space_group().type()
+    #asu_map = asu_map_ext.asymmetric_map(sgt, map_data)
+    #map_data_asu = asu_map.data()
+    #map_data_asu = map_data_asu.shift_origin()
+    #
+    #asu_map = asu_map_ext.asymmetric_map(sgt, map_data_asu,
+    #  self.crystal_gridding.n_real())
+    #f_mask_i = self.f_calc.customized_copy(
+    #  indices = self.f_calc.indices(),
+    #  data    = asu_map.structure_factors(self.f_calc.indices() ))
+
+    def _sf_from_map(map_data):
+      return self.f_calc.structure_factors_from_map(
+        map            = map_data,
+        use_scale      = True,
+        anomalous_flag = False,
+        use_sg         = False)
+
+    #f_mask_i = _sf_from_map(map_data = map_data)
+    #
+    #
+    #self.FV[f_mask_i] = -99
+
+    OFFSET = region.i_seq+1
+    print("<><><><><><><><><><>")
+    dmd = modify(fofc=self.mFoDFc_main, R=2, cutoff=0.3, #0.1/10,
+       crystal_gridding=self.crystal_gridding,
+       mask = self.mask_and_regions.mask_p1)
+    #dmd = map_data
+    #STOP()
+
+    mask_p1 = self.mask_and_regions.mask_p1.deep_copy()
+
+    #mask_ = mmtbx.masks.mask_from_xray_structure(
+    #  xray_structure           = self.xray_structure,
+    #  p1                       = True,
+    #  for_structure_factors    = True,
+    #  solvent_radius           = 0.5,
+    #  shrink_truncation_radius = 0.5,
+    #  n_real                   = self.n_real,
+    #  in_asu                   = False).mask_data
+    #maptbx.unpad_in_place(map=mask_)
+    #dmd = dmd * mask_
+
+    co = maptbx.connectivity(
+      map_data                   = dmd,
+      threshold                  = 0.04, #0.001/10,
+      preprocess_against_shallow = False,
+      wrapping                   = True)
+    if(xray_structure.space_group().type().number() != 1): # not P1
+      co.merge_symmetry_related_regions(
+        space_group = xray_structure.space_group())
+    conn = co.result().as_double()
+    z = zip(co.regions(),range(0,co.regions().size()))
+    sorted_by_volume = sorted(z, key=lambda x: x[0], reverse=True)
+    for i_seq, p in enumerate(sorted_by_volume):
+      i_seq = OFFSET + i_seq
+      v, i = p
+
+      #if(i==0 or i==1): continue
+      if i==0: continue
+
+      step=0.6
+      volume = v*step**3
+      volume_cutoff = 2500
+      if(volume_cutoff is not None and volume < volume_cutoff): continue
+      uc_fraction = v*100./conn.size()
+      print(i, "%3d"%i_seq,"%12.3f"%volume, "%8.4f"%round(uc_fraction,4),
+            "%7s"%str(None), file=log)
+      iselection = (conn==i).iselection()
+      selection = flex.bool(conn.size(), iselection)
+      selection.resize(conn.accessor())
+
+      mask_p1 = mask_p1.set_selected(selection, 0)
+
+      region = group_args(
+        i           = i,
+        i_seq       = i_seq,
+        volume      = volume,
+        uc_fraction = uc_fraction,
+        iselection  = iselection)
+      self.mask_and_regions.regions.append( region )
+
+      dmd_i = dmd.deep_copy()
+      mask_i_asu = dmd_i.set_selected(~selection, 0)
+      #mask_i_asu = mask_i_asu.set_selected(mask_i_asu>0, 1)
+      f_mask_i = _sf_from_map(map_data=mask_i_asu)
+
+      #mask_i_asu = mask_i_asu/flex.max(mask_i_asu)
+      #mask_i_asu = self.mask_and_regions.get_region_asu_mask(region = region, map_data=dmd_i)
+      #mask_i_asu = mask_i_asu/flex.max(mask_i_asu)
+      #
+      #f_mask_i = self.compute_f_mask_i(mask_i_asu)
+      self.FV[f_mask_i] = -1*i_seq
+    print("<><><><><><><><><><>")
+
+
+    fmodel = mmtbx.f_model.manager(
+      f_obs        = self.f_obs,
+      f_calc       = self.f_calc,
+      r_free_flags = self.r_free_flags,
+      f_mask       = _sf_from_map(map_data=mask_p1))
+    fmodel.update_all_scales(remove_outliers=True,
+      apply_scale_k1_to_f_obs = APPLY_SCALE_K1_TO_FOBS)
+    mc = fmodel.electron_density_map().map_coefficients(
+      map_type   = "mFobs-DFmodel",
+      isotropize = False,
+      exclude_free_r_reflections = True)
+    mtz_dataset = mc.as_mtz_dataset(column_root_label = "F")
+    mtz_object = mtz_dataset.mtz_object()
+    mtz_object.write(file_name = "map2.mtz")
+    """
+
+
     #
     # Determine number of secondary regions. Must happen here!
     # Preliminarily if need to do mosaic.
+
     self.n_regions = len(self.FV.values())
+    print("Number of regions (mean(mFo-DFc) >= %s): %d"%(
+      mean_diff_map_threshold, self.n_regions), file=log)
+    if(self.n_regions==0):
+      raise Sorry("Region selection criteria lead to no regions")
     self.do_mosaic = False
     if(self.n_regions>1 and flex.max(self.d_spacings)>6):
       self.do_mosaic = True
-    # Add aggregated small regions (if present)
-    self._add_from_aggregated(selection=small_selection, diff_map=diff_map)
-    # Add aggregated weak map regions (if present)
-    self._add_from_aggregated(selection=weak_selection, diff_map=diff_map)
     # Finalize main Fmask
-    self.f_mask_0 = f_obs.customized_copy(data = f_mask_data_0)
-    # Delete bulk whole mask from memory
-    del self._mask_p1
+    self.f_mask_main = f_obs.customized_copy(data = f_mask_data_main)
+    # Delete large objects from memory
+    del self.diff_map
 
-  def _add_from_aggregated(self, selection, diff_map):
-    if(selection is None or diff_map is None): return
-    self.do_mosaic = True
-    v = selection.count(True)
-    volume = v*self.step**3
-    uc_fraction = v*100./self.conn.size()
-    mask_i = flex.double(flex.grid(self.n_real), 0)
-    mask_i = mask_i.set_selected(selection, 1)
-    diff_map = diff_map.set_selected(diff_map<0,0)
-    mx = flex.mean(diff_map.select((diff_map>0).iselection()))
-    diff_map = diff_map/mx
-    mask_i = mask_i * diff_map
-    mask_i_asu = asu_map_ext.asymmetric_map(
-      self.crystal_symmetry.space_group().type(), mask_i).data()
-    f_mask_i = self.compute_f_mask_i(mask_i_asu)
-    self.FV[f_mask_i] = [round(volume, 3), round(uc_fraction,1)]
+  def _get_sum_f_masks(self):
+    f_masks = list(self.FV.keys())
+    data = f_masks[0].data().deep_copy()
+    for f in f_masks[1:]:
+      data += f.data()
+    return self.f_obs.set().array(data = data)
 
-  def _get_region_i_selection(self, i):
-    if(self._region_i_selection is not None):
-      return self._region_i_selection
-    else:
-      return self.conn==i
+  def _get_map_info(self, iselection):
+    if(self.diff_map is None):
+      return group_args(min=None, max=None, mean=None, sd=None,
+                        string="   None    None    None    None")
+    blob = self.diff_map.select(iselection)
+    mi,ma,me = flex.min(blob), flex.max(blob), flex.mean(blob)
+    sd = blob.sample_standard_deviation()
+    return group_args(min=mi, max=ma, mean=me, sd=sd,
+                      string="%7.3f %7.3f %7.3f %7.3f"%(mi,ma,me,sd))
 
   def _inflate(self, f):
     data = flex.complex_double(self.d_spacings.size(), 0)
     data = data.set_selected(self.sel_gte3, f.data())
     return self.f_obs.set().array(data = data)
 
-  def _compute_f_mask_whole(self):
-    if(self.compute_whole):
-      mask = asu_map_ext.asymmetric_map(
-        xray_structure.crystal_symmetry().space_group().type(),
-        self._mask_p1).data()
-      return self._inflate(
-        self.miller_array.structure_factors_from_asu_map(
-          asu_map_data = mask, n_real = self.n_real))
-    else: return None
-
-  def _compute_mask_in_p1(self):
-    mask = mmtbx.masks.mask_from_xray_structure(
-      xray_structure           = self.xray_structure,
-      p1                       = True,
-      for_structure_factors    = True,
-      solvent_radius           = self.r_sol,
-      shrink_truncation_radius = self.r_shrink,
-      n_real                   = self.n_real,
-      in_asu                   = False).mask_data
-    maptbx.unpad_in_place(map=mask)
-    if(self.write_masks):
-      write_map_file(
-        crystal_symmetry = self.crystal_symmetry, # Is this correct symmetry?
-        map_data         = mask,
-        file_name        = "mask_whole.mrc")
-    return mask
-
   def compute_f_mask_i(self, mask_i_asu):
     return self._inflate(self.miller_array.structure_factors_from_asu_map(
       asu_map_data = mask_i_asu, n_real = self.n_real))
 
-  def compute_diff_map(self, f_mask_data_0):
+  def compute_diff_map(self, f_mask_data):
     if(self.f_calc is None): return None
-    f_mask = self.f_obs.customized_copy(data = f_mask_data_0)
+    f_mask = self.f_obs.customized_copy(data = f_mask_data)
     fmodel = mmtbx.f_model.manager(
-      f_obs  = self.f_obs,
-      f_calc = self.f_calc,
-      f_mask = f_mask)
+      f_obs        = self.f_obs,
+      f_calc       = self.f_calc,
+      r_free_flags = self.r_free_flags,
+      f_mask       = f_mask)
     fmodel.update_all_scales(remove_outliers=True,
       apply_scale_k1_to_f_obs = APPLY_SCALE_K1_TO_FOBS)
-    self.mFoDFc_0 = fmodel.electron_density_map().map_coefficients(
+    self.mFoDFc_main = fmodel.electron_density_map().map_coefficients(
       map_type   = "mFobs-DFmodel",
-      isotropize = True,
-      exclude_free_r_reflections = False)
-    fft_map = self.mFoDFc_0.fft_map(crystal_gridding = self.crystal_gridding)
+      isotropize = False,
+      exclude_free_r_reflections = True)
+    fft_map = self.mFoDFc_main.fft_map(crystal_gridding = self.crystal_gridding)
     fft_map.apply_sigma_scaling()
     return fft_map.real_map_unpadded()
 
-  def compute_i_mask_asu(self, selection, volume):
-    mask_i = flex.double(flex.grid(self.n_real), 0)
-    mask_i = mask_i.set_selected(selection, 1)
-    if(self.write_masks):
-      write_map_file(
-        crystal_symmetry = self.crystal_symmetry,
-        map_data         = mask_i,
-        file_name        = "mask_%s.mrc"%str(round(volume,3)))
-    return asu_map_ext.asymmetric_map(
-      self.crystal_symmetry.space_group().type(), mask_i).data()
-
 def algorithm_0(f_obs, F, kt):
   """
   Grid search
@@ -803,43 +1041,76 @@ def algorithm_0(f_obs, F, kt):
   r = [1,]+r
   return r
 
-def algorithm_2(i_obs, F, x, use_curvatures=True, macro_cycles=10):
+def algorithm_2(i_obs, F, x, use_curvatures, use_lbfgsb, macro_cycles=10,
+                max_iterations=100):
   """
   Unphased one-step search
   """
-  calculator = tg(i_obs = i_obs, F=F, x = x, use_curvatures=use_curvatures)
+  assert use_curvatures in [True, False, None]
+  assert use_lbfgsb in [True, False, None]
+  upper = flex.double([1.1] + [12]*(x.size()-1))
+  lower = flex.double([0.9] + [0]*(x.size()-1))
   for it in range(macro_cycles):
-    if(use_curvatures):
-      m = minimizer(max_iterations=100, calculator=calculator)
-    else:
-      #upper = flex.double([1.1] + [1]*(x.size()-1))
-      #lower = flex.double([0.9] + [-1]*(x.size()-1))
+    x = x.set_selected(x<0,1.e-6)
+    if(use_curvatures is True): # Curvatures only
+      calculator = tg(i_obs = i_obs, F=F, x = x, use_curvatures=True)
+      m = minimizer2(
+        max_iterations=max_iterations, calculator=calculator).run(use_curvatures=True)
+      x = m.x
+      x = x.set_selected(x<0,1.e-6)
+    elif(use_curvatures is False): # No curvatures at all
+      calculator = tg(i_obs = i_obs, F=F, x = x, use_curvatures=False)
+      if(use_lbfgsb is True):
+        m = tncs.minimizer(
+          potential       = calculator,
+          use_bounds      = 2,
+          lower_bound     = lower,
+          upper_bound     = upper,
+          max_iterations  = max_iterations,
+          initial_values  = x).run()
+      else:
+        m = minimizer(max_iterations=max_iterations, calculator=calculator)
+      x = m.x
+      x = x.set_selected(x<0,1.e-6)
+    elif(use_curvatures is None):
+      calculator = tg(i_obs = i_obs, F=F, x = x, use_curvatures=True)
+      m = minimizer2(
+        max_iterations=max_iterations, calculator=calculator).run(use_curvatures=True)
+      x = m.x
+      x = x.set_selected(x<0,1.e-6)
+      calculator = tg(i_obs = i_obs, F=F, x = x, use_curvatures=False)
+      if(use_lbfgsb is True):
+        m = tncs.minimizer(
+          potential       = calculator,
+          use_bounds      = 2,
+          lower_bound     = lower,
+          upper_bound     = upper,
+          max_iterations  = max_iterations,
+          initial_values  = x).run()
+      else:
+        m = minimizer(max_iterations=max_iterations, calculator=calculator)
+      x = m.x
+      x = x.set_selected(x<0,1.e-6)
 
-      upper = flex.double([1.1] + [5]*(x.size()-1))
-      lower = flex.double([0.9] + [-5]*(x.size()-1))
+#  calculator = tg(i_obs = i_obs, F=F, x = m.x, use_curvatures=True)
+#  m = minimizer2(max_iterations=100, calculator=calculator).run(use_curvatures=True)
+#
+#  calculator = tg(i_obs = i_obs, F=F, x = m.x, use_curvatures=False)
+#  m = tncs.minimizer(
+#        potential       = calculator,
+#        use_bounds      = 2,
+#        lower_bound     = lower,
+#        upper_bound     = upper,
+#        max_iterations  = 100,
+#        initial_values  = calculator.x).run()
 
-      #upper = flex.double([10] + [5]*(x.size()-1))
-      #lower = flex.double([0.1] + [-5]*(x.size()-1))
-      #upper = flex.double([10] + [0.65]*(x.size()-1))
-      #lower = flex.double([0.1] + [0]*(x.size()-1))
-
-      #upper = flex.double([1] + [0.65]*(x.size()-1))
-      #lower = flex.double([1] + [0]*(x.size()-1))
-      #upper = flex.double([1] + [5.65]*(x.size()-1))
-      #lower = flex.double([1] + [-5]*(x.size()-1))
-      m = tncs.minimizer(
-        potential       = calculator,
-        use_bounds      = 2,
-        lower_bound     = lower,
-        upper_bound     = upper,
-        initial_values  = x).run()
-    calculator = tg(i_obs = i_obs, F=F, x = m.x, use_curvatures=use_curvatures)
-  if(use_curvatures):
-    for it in range(10):
-      m = minimizer(max_iterations=100, calculator=calculator)
-      calculator = tg(i_obs = i_obs, F=F, x = m.x, use_curvatures=use_curvatures)
-      m = minimizer2(max_iterations=100, calculator=calculator).run(use_curvatures=True)
-      calculator = tg(i_obs = i_obs, F=F, x = m.x, use_curvatures=use_curvatures)
+  #if(use_curvatures):
+  #  for it in range(10):
+  #    m = minimizer(max_iterations=100, calculator=calculator)
+  #    calculator = tg(i_obs = i_obs, F=F, x = m.x, use_curvatures=use_curvatures)
+  #    m.x = m.x.set_selected(m.x<0,1.e-3)
+  #    m = minimizer2(max_iterations=100, calculator=calculator).run(use_curvatures=True)
+  #    calculator = tg(i_obs = i_obs, F=F, x = m.x, use_curvatures=use_curvatures)
   return m.x
 
 def algorithm_3(i_obs, fc, f_masks):
@@ -900,52 +1171,66 @@ def algorithm_3(i_obs, fc, f_masks):
   return [math.exp(x) for x in lnK]
 
 def algorithm_4(f_obs, F, phase_source, max_cycles=100, auto_converge_eps=1.e-7,
-                use_cpp=True):
+                use_cpp=True, x_init=None):
   """
   Phased simultaneous search (alg4)
   """
   fc, f_masks = F[0], F[1:]
+  if(x_init is not None):
+    assert x_init.size() == len(f_masks)
+    # Apply scale
+    tmp = []
+    for fm, s in zip(f_masks, x_init):
+      fm = fm.array(data = fm.data()*s)
+      tmp.append(fm)
+    f_masks = tmp
+  #
   fc = fc.deep_copy()
-  F = [fc]+F[1:]
+  F = [fc]+f_masks
   # C++ version
   if(use_cpp):
-    return mosaic_ext.alg4(
+    result = mosaic_ext.alg4(
       [f.data() for f in F],
       f_obs.data(),
       phase_source.data(),
       max_cycles,
       auto_converge_eps)
-  # Python version (1.2-3 times slower, but much more readable!)
-  cntr = 0
-  x_prev = None
-  while True:
-    f_obs_cmpl = f_obs.phase_transfer(phase_source = phase_source)
-    A = []
-    b = []
-    for j, Fj in enumerate(F):
-      A_rows = []
-      for n, Fn in enumerate(F):
-        Gjn = flex.real( Fj.data()*flex.conj(Fn.data()) )
-        A_rows.append( flex.sum(Gjn) )
-      Hj = flex.real( Fj.data()*flex.conj(f_obs_cmpl.data()) )
-      b.append(flex.sum(Hj))
-      A.extend(A_rows)
-    A = matrix.sqr(A)
-    A_1 = A.inverse()
-    b = matrix.col(b)
-    x = A_1 * b
-    #
-    fc_d = flex.complex_double(phase_source.indices().size(), 0)
-    for i, f in enumerate(F):
-      fc_d += f.data()*x[i]
-    phase_source = phase_source.customized_copy(data = fc_d)
-    x_ = x[:]
-    #
-    cntr+=1
-    if(cntr>max_cycles): break
-    if(x_prev is None): x_prev = x_[:]
-    else:
-      max_diff = flex.max(flex.abs(flex.double(x_prev)-flex.double(x_)))
-      if(max_diff<=auto_converge_eps): break
-      x_prev = x_[:]
-  return x_
+  else:
+    # Python version (1.2-3 times slower, but much more readable!)
+    cntr = 0
+    x_prev = None
+    while True:
+      f_obs_cmpl = f_obs.phase_transfer(phase_source = phase_source)
+      A = []
+      b = []
+      for j, Fj in enumerate(F):
+        A_rows = []
+        for n, Fn in enumerate(F):
+          Gjn = flex.real( Fj.data()*flex.conj(Fn.data()) )
+          A_rows.append( flex.sum(Gjn) )
+        Hj = flex.real( Fj.data()*flex.conj(f_obs_cmpl.data()) )
+        b.append(flex.sum(Hj))
+        A.extend(A_rows)
+      A = matrix.sqr(A)
+      A_1 = A.inverse()
+      b = matrix.col(b)
+      x = A_1 * b
+      #
+      fc_d = flex.complex_double(phase_source.indices().size(), 0)
+      for i, f in enumerate(F):
+        fc_d += f.data()*x[i]
+      phase_source = phase_source.customized_copy(data = fc_d)
+      x_ = x[:]
+      #
+      cntr+=1
+      if(cntr>max_cycles): break
+      if(x_prev is None): x_prev = x_[:]
+      else:
+        max_diff = flex.max(flex.abs(flex.double(x_prev)-flex.double(x_)))
+        if(max_diff<=auto_converge_eps): break
+        x_prev = x_[:]
+    result = x_
+  if(x_init is not None):
+    for i,tmp in enumerate(x_init):
+      result[i+1] = result[i+1] * x_init[i]
+  return result
