@@ -34,7 +34,8 @@ master_phil = libtbx.phil.parse("""
   ccp4_map_file = None
     .help = Input map file (CCP4/mrc format).
     .short_caption = Input map file
-    .type = str
+    .type = path
+    .style = file_type:ccp4_map bold input_file
   mask_file_name = None
     .help = Input mask file (CCP4/mrc format).
     .short_caption = Input mask file
@@ -182,7 +183,7 @@ master_phil = libtbx.phil.parse("""
   increase_box_cushion_and_atom_radius_for_soft_mask = True
     .type = bool
     .help = Expand cushion and atom radii by soft_mask_radius
-    .short_caption = Increase box cusion and atom radius for soft mask
+    .short_caption = Increase box cushion and atom radius for soft mask
 
   soft_mask_extract_unique = True
     .type = bool
@@ -370,6 +371,13 @@ master_phil = libtbx.phil.parse("""
     .help = Remove all output map labels
     .short_caption = Remove labels
 
+  invert_hand = False
+    .type = bool
+    .help = Just before writing out the map, swap the order of all sections \
+             in Z.  This will change the hand of the map. Note that this\
+             removes any correspondence to models (these are not inverted).
+    .short_caption = Invert hand of map
+
   gui
     .help = "GUI-specific parameter required for output directory"
   {
@@ -558,10 +566,9 @@ def check_parameters(inputs = None, params = None,
       raise Sorry("Please set resolution for extract_unique")
     if (not params.symmetry) and (not params.symmetry_file) and \
         (not ncs_object):
-      raise Sorry(
-        "Please supply a symmetry file or symmetry for extract_unique (you "+
-       "\ncan try symmetry = ALL if you do not know your symmetry or "+
-        "symmetry = C1 if \nthere is none)")
+      params.symmetry="ALL"
+      print("Setting symmetry=ALL as no symmetry information supplied",
+        file = log)
     if params.mask_atoms:
       raise Sorry("You cannot set mask_atoms with extract_unique")
 
@@ -683,6 +690,13 @@ def modify_params(params = None,
     params = get_origin_or_bounds_from_ccp4_file(params = params, log = log)
 
 
+  # Check that bounds are None or tuples of three
+
+  if params.lower_bounds and len(params.lower_bounds) != 3:
+    raise Sorry("Need 3 values for lower_bounds")
+  if params.upper_bounds and len(params.upper_bounds) != 3:
+    raise Sorry("Need 3 values for upper_bounds")
+
   if params.output_origin_grid_units is not None and params.keep_origin:
     params.keep_origin = False
     print("Setting keep_origin = False as output_origin_grid_units is set",
@@ -848,6 +862,9 @@ def print_notes(params = None,
     print("Map unit cell as grid units:  (%s, %s, %s)" %(
       tuple(ccp4_map.map_data().all())), file = log)
 
+    if params.invert_hand:
+      print("\nOutput map will be inverted hand "+
+         "(swapping order of sections in Z)", file = log)
 
 def run(args,
      ncs_object = None,  # ncs object
@@ -1134,6 +1151,9 @@ def run(args,
   if params.remove_output_map_labels:
     mam.map_manager().remove_labels()
 
+  if params.invert_hand:
+    mam.map_manager().invert_hand()
+
   # Print out any notes about the output files
   print_notes(params = params, mam = mam,
     crystal_symmetry = crystal_symmetry,
