@@ -6,7 +6,9 @@ from scitbx.math import dihedral_angle
 from mmtbx.ligands.ready_set_basics import construct_xyz
 from mmtbx.ligands.ready_set_basics import generate_atom_group_atom_names
 from mmtbx.ligands.ready_set_basics import get_hierarchy_h_atom
-from mmtbx.hydrogens.specialised_hydrogen_atoms import conditional_add_cys_hg_to_atom_group
+from mmtbx.ligands.ready_set_basics import get_proton_info
+from mmtbx.ligands.hierarchy_utils import _add_atom_to_chain
+from mmtbx.hydrogens.specialised_hydrogen_atoms import process_disulphide_hydrogen_atoms
 from six.moves import range
 
 get_class = iotbx.pdb.common_residue_names_get_class
@@ -19,27 +21,6 @@ def is_n_terminal_residue(residue_group):
   #if residues[0] in n_terminal_amino_acid_codes: return True
   return False
 
-def _add_atom_to_chain(atom, ag):
-  rg = _add_atom_to_residue_group(atom, ag)
-  chain = ag.parent().parent()
-  tc = iotbx.pdb.hierarchy.chain()
-  tc.id = chain.id
-  tc.append_residue_group(rg)
-  return tc
-
-def _add_atom_to_residue_group(atom, ag):
-  tag = iotbx.pdb.hierarchy.atom_group()
-  tag.resname = ag.resname
-  tag.append_atom(atom)
-  rg = iotbx.pdb.hierarchy.residue_group()
-  rg.resseq = ag.parent().resseq
-  rg.append_atom_group(tag)
-  for i, c in enumerate(letters):
-    if c==ag.parent().parent().id:
-      break
-  atom.tmp = i
-  return rg
-
 def _add_atoms_from_chains_to_end_of_hierarchy(hierarchy, chains):
   lookup = {}
   for chain in chains:
@@ -79,7 +60,8 @@ def add_n_terminal_hydrogens_to_atom_group(ag,
         ns.append(key)
     if len(ns)>=3: return rc
 
-  atom = ag.get_atom('H')
+  proton_element, proton_name = get_proton_info(ag)
+  atom = ag.get_atom(proton_element) # just so happens that the atom is named H/D
   dihedral=120.
   if atom:
     dihedral = dihedral_angle(sites=[atom.xyz,
@@ -90,50 +72,69 @@ def add_n_terminal_hydrogens_to_atom_group(ag,
                               deg=True)
   if retain_original_hydrogens: pass
   else:
-    if ag.get_atom("H"): # maybe needs to be smarter or actually work
-      ag.remove_atom(ag.get_atom('H'))
+    if ag.get_atom(proton_name): # maybe needs to be smarter or actually work
+      ag.remove_atom(ag.get_atom(proton_name))
   #if use_capping_hydrogens and 0:
   #  for i, atom in enumerate(ag.atoms()):
   #    if atom.name == ' H3 ':
   #      ag.remove_atom(i)
   #      break
   # add H1
-  rh3 = construct_xyz(n, 0.9,
+  rh3 = construct_xyz(n, 1.0,
                       ca, 109.5,
                       c, dihedral,
                      )
   # this could be smarter
-  possible = ['H', 'H1', 'H2', 'H3', 'HT1', 'HT2']
+  if proton_element=='H':
+    possible = ['H', 'H1', 'H2', 'H3', 'HT1', 'HT2']
+  elif proton_element=='D':
+    possible = ['D', 'D1', 'D2', 'D3'] #, 'HT1', 'HT2']
   h_count = 0
+  d_count = 0
   for h in possible:
     if ag.get_atom(h): h_count+=1
+    if ag.get_atom(h.replace('H', 'D')): d_count+=1
   number_of_hydrogens=3
   if use_capping_hydrogens:
     number_of_hydrogens-=1
     #if ag.atoms()[0].parent().resname=='PRO':
     #  number_of_hydrogens=-1
     #  # should name the hydrogens correctly
-  if h_count>=number_of_hydrogens: return []
+  if h_count+d_count>=number_of_hydrogens: return []
+  def distance2(xyz1, xyz2):
+    d2=0
+    for k in range(3):
+      d2+=(xyz2[k]-xyz1[k])**2
+    return d2
+  j=0
   for i in range(0, number_of_hydrogens):
-    name = " H%d " % (i+1)
+    name = " %s%d " % (proton_element, i+1)
     if retain_original_hydrogens:
-      if i==0 and ag.get_atom('H'): continue
+      if i==0 and ag.get_atom(proton_name): continue
+      if i==1 and ag.get_atom(proton_name):
+        retained = ag.get_atom(proton_name)
+        d2 = distance2(retained.xyz, rh3[j])
+        if d2<0.5: j+=1
     if ag.get_atom(name.strip()): continue
     if ag.resname=='PRO':
       if i==0:
         continue
     atom = iotbx.pdb.hierarchy.atom()
     atom.name = name
-    atom.element = "H"
-    atom.xyz = rh3[i]
+    atom.element = proton_element
+    atom.xyz = rh3[j]
     atom.occ = n.occ
     atom.b = n.b
     atom.segid = ' '*4
     if append_to_end_of_model and i+1==number_of_hydrogens:
-      rg = _add_atom_to_chain(atom, ag)
+      rg = _add_atom_to_chain(atom,
+                              ag,
+                              icode=n.parent().parent().icode)
       rc.append(rg)
     else:
       ag.append_atom(atom)
+    j+=1
+    if j==number_of_hydrogens: j=0
   return rc
 
 def add_n_terminal_hydrogens_to_residue_group(residue_group,
@@ -169,14 +170,15 @@ def add_c_terminal_oxygens_to_atom_group(ag,
   #
   # do we need ANISOU
   #
+  proton_element, proton_name = get_proton_info(ag)
   rc = []
   atom_name=' OXT'
   atom_element = 'O'
   bond_length=1.231
   if use_capping_hydrogens:
     if ag.get_atom(atom_name.strip()): return []
-    atom_name=" HC "
-    atom_element="H"
+    atom_name=" %sC " % proton_element
+    atom_element=proton_element
     bond_length=1.
   if ag.get_atom(atom_name.strip()): return []
   if c_ca_n is not None:
@@ -215,7 +217,9 @@ def add_c_terminal_oxygens_to_atom_group(ag,
       atom.segid = ' '*4
       atom.xyz = ro2[i]
       if append_to_end_of_model:
-        chain = _add_atom_to_chain(atom, ag)
+        chain = _add_atom_to_chain(atom,
+                                   ag,
+                                   icode=c.parent().parent().icode)
         rc.append(chain)
       else:
         # add the atom to the hierarchy
@@ -227,7 +231,7 @@ def add_c_terminal_oxygens_to_residue_group(residue_group,
                                             use_capping_hydrogens=False,
                                             append_to_end_of_model=False,
                                            ):
-  rc=[]
+  result=[]
   for ag, rc in generate_atom_group_atom_names(residue_group,
                                                        ['C', 'CA', 'N'],
                                                        ):
@@ -240,8 +244,8 @@ def add_c_terminal_oxygens_to_residue_group(residue_group,
       append_to_end_of_model=append_to_end_of_model,
       c_ca_n = [c, ca, n],
     )
-    rc += tmp
-  return rc
+    result += tmp
+  return result
 
 def add_main_chain_o_to_atom_group(ag, c_ca_n=None):
   # cetral functuon
@@ -314,77 +318,77 @@ def add_main_chain_atoms_to_protein_three(three):
 
   assert 0
 
-# def generate_residues_via_conformer(hierarchy,
-#                                     backbone_only=False,
-#                                     verbose=False,
-#                                     ):
-#   assert 0
-#   backbone_asc = hierarchy.atom_selection_cache()
-#   backbone_sel = backbone_asc.selection("name ca or name c or name n or name o or name cb")
-#   backbone_hierarchy = hierarchy.select(backbone_sel)
-#   get_class = iotbx.pdb.common_residue_names_get_class
-#   loop_hierarchy=hierarchy
-#   if backbone_only: loop_hierarchy=backbone_hierarchy
-#   for model in loop_hierarchy.models():
-#     if verbose: print 'model: "%s"' % model.id
-#     for chain in model.chains():
-#       if verbose: print 'chain: "%s"' % chain.id
-#       for conformer in chain.conformers():
-#         if verbose: print '  conformer: altloc="%s"' % (
-#           conformer.altloc)
-# #        while threes: del threes[0]
-# #        threes.start=None
-# #        threes.end=None
-# #        list_of_threes = []
-#         for residue in conformer.residues():
-#           if verbose:
-#             if residue.resname not in ["HOH"]:
-#               print '    residue: resname="%s" resid="%s"' % (
-#                 residue.resname, residue.resid())
-#           if verbose: print '      residue class : %s' % get_class(residue.resname)
-#           if get_class(residue.resname) not in ["common_amino_acid",
-#                                                 'modified_amino_acid',
-#                                               ]:
-#             # this needs to be moved to cctbx get_class
-#             #'ETA', # COOH terminal - not in modified
-#             if residue.resname not in aac.three_letter_l_given_three_letter_d:
-#               continue
-#           yield residue
+def generate_residues_via_conformer(hierarchy,
+                                    backbone_only=False,
+                                    verbose=False,
+                                    ):
+  backbone_asc = hierarchy.atom_selection_cache()
+  backbone_sel = backbone_asc.selection("name ca or name c or name n or name o or name cb")
+  backbone_hierarchy = hierarchy.select(backbone_sel)
+  get_class = iotbx.pdb.common_residue_names_get_class
+  loop_hierarchy=hierarchy
+  if backbone_only: loop_hierarchy=backbone_hierarchy
+  for model in loop_hierarchy.models():
+    if verbose: print('model: "%s"' % model.id)
+    for chain in model.chains():
+      if verbose: print('chain: "%s"' % chain.id)
+      for conformer in chain.conformers():
+        if verbose: print('  conformer: altloc="%s"' % (conformer.altloc))
+#        while threes: del threes[0]
+#        threes.start=None
+#        threes.end=None
+#        list_of_threes = []
+        for residue in conformer.residues():
+          if verbose:
+            if residue.resname not in ["HOH"]:
+              print('    residue: resname="%s" resid="%s"' % (
+                residue.resname, residue.resid()))
+          if verbose: print('      residue class : %s' % get_class(residue.resname))
+          if get_class(residue.resname) not in ["common_amino_acid",
+                                                'modified_amino_acid',
+                                              ]:
+            # this needs to be moved to cctbx get_class
+            #'ETA', # COOH terminal - not in modified
+            if residue.resname not in aac.three_letter_l_given_three_letter_d:
+              continue
+          yield residue
 
-# def generate_protein_fragments(hierarchy,
-#                                geometry,
-#                                backbone_only=False,
-#                                use_capping_hydrogens=False,
-#                                verbose=False,
-#                                ):
-#   assert 0
-#   from mmtbx.conformation_dependent_library.multi_residue_class import \
-#     ThreeProteinResidues, RestraintsRegistry
-#   registry = RestraintsRegistry()
-#   threes = ThreeProteinResidues(geometry, registry=registry)
-#   for residue in generate_residues_via_conformer(hierarchy,
-#                                                  backbone_only=backbone_only,
-#                                                  verbose=verbose,
-#                                                  ):
-#     list.append(threes, residue)
-#     if verbose: print 'THREE',threes
-#     sub_unit = threes.provide_second_sub_unit_if_unlinked()
-#     if verbose: print 'THREE, SUBUNIT',threes, sub_unit
-#     if sub_unit:
-#       threes.start = True
-#       threes.end = True
-#       yield threes
-#       threes = sub_unit
-#   threes.start = True
-#   threes.end = True
-#   yield threes
+def generate_protein_fragments(hierarchy,
+                               geometry,
+                               backbone_only=False,
+                               use_capping_hydrogens=False,
+                               verbose=False,
+                               ):
+  '''
+  Called by qrefine. This should be replaced by the function in hierarchy_utils
+  after much testing.
+  '''
+  from mmtbx.conformation_dependent_library.multi_residue_class import \
+    ThreeProteinResidues, RestraintsRegistry
+  registry = RestraintsRegistry()
+  threes = ThreeProteinResidues(geometry, registry=registry)
+  for residue in generate_residues_via_conformer(hierarchy,
+                                                 backbone_only=backbone_only,
+                                                 verbose=verbose,
+                                                 ):
+    list.append(threes, residue)
+    if verbose: print('THREE',threes)
+    sub_unit = threes.provide_second_sub_unit_if_unlinked()
+    if verbose: print('THREE, SUBUNIT',threes, sub_unit)
+    if sub_unit:
+      threes.start = True
+      threes.end = True
+      yield threes
+      threes = sub_unit
+  threes.start = True
+  threes.end = True
+  yield threes
 
 def _hierarchy_into_slots(hierarchy,
                           geometry_restraints_manager,
                           verbose=False,
                           ):
   def _is_linked(residue_group1, residue_group2, bpt):
-    print(residue_group1, residue_group2)
     if residue_group2 is None: return False
     if residue_group2 is False: return False
     atom_group1 = residue_group1.only_atom_group()
@@ -426,9 +430,12 @@ def _hierarchy_into_slots(hierarchy,
         slots.append(False)
         continue
       if slots:
-        if _is_linked(residue_group,
-                      slots[-1],
-                      geometry_restraints_manager.bond_params_table):
+        bonded = _is_linked(residue_group,
+                            slots[-1],
+                            geometry_restraints_manager.bond_params_table)
+        if getattr(bonded, 'origin_id', None)!=0: # default for peptide links...
+          bonded=False
+        if bonded:
           pass
         else:
           slots.append(None)
@@ -439,7 +446,6 @@ def _hierarchy_into_slots(hierarchy,
 def generate_residue_group_with_start_and_end(hierarchy,
                                               geometry_restraints_manager,
                                               # ideal_hierarchy=None,
-                                              verbose=False,
                                               ):
   # assert not ideal_hierarchy
   slots = _hierarchy_into_slots(hierarchy, geometry_restraints_manager)
@@ -570,11 +576,12 @@ def add_terminal_hydrogens_via_residue_groups(hierarchy,
   for residue_group, start, end in generate_residue_group_with_start_and_end(
     hierarchy,
     geometry_restraints_manager,
-    verbose=verbose,
     ):
     if use_capping_hydrogens:
-      conditional_add_cys_hg_to_atom_group(geometry_restraints_manager,
-                                           residue_group)
+      # conditional_add_cys_hg_to_atom_group(geometry_restraints_manager,
+      #                                      residue_group)
+      process_disulphide_hydrogen_atoms(geometry_restraints_manager,
+                                        residue_group)
     if start:
       # ptr+=1
       # assert ptr==1
@@ -654,7 +661,7 @@ def delete_charged_n_terminal_hydrogens(hierarchy):
 
 def add_water_hydrogen_atoms_simple(hierarchy, log=None):
   for atom_group in hierarchy.atom_groups():
-    if atom_group.resname=='HOH':
+    if atom_group.resname in ['HOH', 'DOD']:
       if len(atom_group.atoms())==3: continue
       o_atom = atom_group.atoms()[0]
       xyz = (o_atom.xyz[0]+1, o_atom.xyz[1], o_atom.xyz[2])
@@ -750,42 +757,3 @@ def main_hydrogen(model,
       verbose=False,
       )
   return
-
-  assert 0
-
-
-  n_done = []
-  for three in hierarchy_utils.generate_protein_fragments(
-    hierarchy,
-    geometry_restraints_manager,
-    backbone_only=False,
-    #use_capping_hydrogens=use_capping_hydrogens,
-    ):
-    if verbose: print(three)
-    if len(three)==1: continue
-    for i, residue in enumerate(three):
-      if not i: continue
-      # this may not be necessary with the new threes
-      residue = hierarchy_utils.get_residue_group(residue, atoms)
-      h = hierarchy_utils.get_atom_from_residue_group(residue, 'H')
-      if h is None:
-        assert 0
-        for ag, (n, ca, c) in ready_set_basics.generate_atom_group_atom_names(
-            residue,
-            ['N', 'CA', 'C'],
-        ):
-          if ag.resname in ['PRO']: continue
-          if n in n_done: continue
-          n_done.append(n)
-          dihedral = 0
-          rh3 = general_utils.construct_xyz(n, 0.9,
-                                            ca, 109.5,
-                                            c, dihedral,
-          )
-          atom = create_atom(' H  ', 'H', rh3[0], n)
-          # adding to atom_group
-          # need to add to geometry_restraints_manager
-          ag.append_atom(atom)
-          if verbose: print(atom.quote())
-          assert ag.resname!='PRO'
-
