@@ -123,7 +123,15 @@ class geometry(object):
     if(self.from_restraints is not None):
       mi,ma,me = self.from_restraints.planarity_deviations()
       n = self.from_restraints.get_filtered_n_planarity_proxies()
-    return group_args(min = mi, max = ma, mean = me, n = n)
+    pp = protein_planes(hierarchy = self.pdb_hierarchy)
+    asc = self.pdb_hierarchy.atom_selection_cache()
+    sel = asc.selection("element H or element D")
+    pp_noH = None
+    if sel.count(True) > 0:
+      pp_noH = protein_planes(hierarchy = self.pdb_hierarchy.select(~sel))
+    return group_args(min = mi, max = ma, mean = me, n = n,
+      protein_planes_max_dev = pp,
+      protein_planes_max_dev_noH = pp_noH)
 
   def parallelity(self):
     mi,ma,me,n = 0,0,0,0
@@ -158,6 +166,8 @@ class geometry(object):
           outliers_only = False)
     return group_args(
       outliers = self.cached_rota.percent_outliers,
+      favored  = self.cached_rota.percent_favored,
+      allowed  = self.cached_rota.percent_allowed,
       rotalyze = self.cached_rota #XXX Bulky object -- REMOVE!
       )
 
@@ -274,9 +284,9 @@ class geometry(object):
     r = self.result()
     f="bond: %6.3f angle: %6.2f dihedral: %6.2f"
     if assert_zero:
-      assert abs(r.bond.mean)<1e-3, 'bond rmsd is not zero'
-      assert abs(r.angle.mean)<1e-3, 'angle rmsd is not zero'
-      assert abs(r.dihedral.mean)<1e-3, 'dihedral rmsd is not zero'
+      assert abs(r.bond.mean)<1e-3, 'bond rmsd is not zero : %0.5f' % r.bond.mean
+      assert abs(r.angle.mean)<1e-3, 'angle rmsd is not zero : %0.5f' % r.angle.mean
+      assert abs(r.dihedral.mean)<1e-3, 'dihedral rmsd is not zero : %0.5f' % r.dihedral.mean
     return f%(r.bond.mean, r.angle.mean, r.dihedral.mean)
 
   def show_planarity_details(self):
@@ -286,11 +296,22 @@ class geometry(object):
 
   def show_short(self):
     r = self.result()
-    f="bond: %6.3f angle: %6.2f clash: %5.1f rota: %5.2f rama: f: %6.2f o: %6.2f Z: %6.2f cb: %6.2f"
-    return f%(r.bond.mean, r.angle.mean, r.clash.score, r.rotamer.outliers,
-      r.ramachandran.favored, r.ramachandran.outliers, r.rama_z.whole.value, r.c_beta.outliers)
+    f="%s %s %s %s %s %s %s %s"
+    return f%(
+      format_value("bond: %6.3f",r.bond.mean),
+      format_value("angle: %6.2f", r.angle.mean),
+      format_value("clash: %5.1f", r.clash.score),
+      format_value("rota: %5.2f", r.rotamer.outliers),
+      format_value("rama: f: %6.2f", r.ramachandran.favored),
+      format_value("o: %6.2f", r.ramachandran.outliers),
+      format_value("Z: %6.2f", r.rama_z.whole.value),
+      format_value("cb: %6.2f", r.c_beta.outliers) )
 
-  def show(self, log=None, prefix="", uppercase=True):
+  def number_of_bonds(self):
+    bonds = self.bond()
+    return bonds.n
+
+  def show(self, log=None, prefix="", exclude_protein_only_stats=False, uppercase=True):
     if(log is None): log = sys.stdout
     def fmt(f1,f2,d1,z1=None):
       if f1 is None  : return '   -       -       -  '
@@ -307,16 +328,17 @@ class geometry(object):
     a,b,c,d,p,n = res.angle, res.bond, res.chirality, res.dihedral, \
       res.planarity, res.nonbonded
     az, bz = res.angle_z, res.bond_z
-    result = """
+    result = """%s
 %sGeometry Restraints Library: %s
-%sDeviations from Ideal Values - rmsd. rmsZ for bonds and angles.
+%sDeviations from Ideal Values - rmsd, rmsZ for bonds and angles.
 %s  Bond      : %s
 %s  Angle     : %s
 %s  Chirality : %s
 %s  Planarity : %s
 %s  Dihedral  : %s
 %s  Min Nonbonded Distance : %s
-%s"""%(prefix,
+%s"""%(prefix.strip(),
+       prefix,
        self.restraints_source,
        prefix,
        prefix, fmt(b.mean, b.max, b.n, bz.mean),
@@ -326,14 +348,18 @@ class geometry(object):
        prefix, fmt(d.mean, d.max, d.n),
        prefix, fmt2(n.min).strip(),
        prefix)
-    result += """
+    if not exclude_protein_only_stats:
+      result += """
 %sMolprobity Statistics.
 %s  All-atom Clashscore : %s
 %s  Ramachandran Plot:
 %s    Outliers : %5.2f %%
 %s    Allowed  : %5.2f %%
 %s    Favored  : %5.2f %%
-%s  Rotamer Outliers : %5.2f %%
+%s  Rotamer:
+%s    Outliers : %5.2f %%
+%s    Allowed  : %5.2f %%
+%s    Favored  : %5.2f %%
 %s  Cbeta Deviations : %5.2f %%
 %s  Peptide Plane:
 %s    Cis-proline     : %s %%
@@ -346,16 +372,41 @@ class geometry(object):
         prefix, res.ramachandran.outliers,
         prefix, res.ramachandran.allowed,
         prefix, res.ramachandran.favored,
+        prefix,
         prefix, res.rotamer.outliers,
+        prefix, res.rotamer.allowed,
+        prefix, res.rotamer.favored,
         prefix, res.c_beta.outliers,
         prefix,
         prefix, format_value("%5.2f", res.omega.cis_proline).strip(),
         prefix, format_value("%5.2f", res.omega.cis_general).strip(),
         prefix, format_value("%5.2f", res.omega.twisted_proline).strip(),
         prefix, format_value("%5.2f", res.omega.twisted_general).strip())
-    result += """
+      result += """
 %s"""%prefix
-    result += res.rama_z.as_string(prefix=prefix)
+      result += res.rama_z.as_string(prefix=prefix)
+    #
+    if p.protein_planes_max_dev:
+      result += "\n\n%sMax deviation from planes:"%prefix
+      result += "\n%s   Type  MaxDev  MeanDev LineInFile"%prefix
+      for pp in p.protein_planes_max_dev:
+        result += "\n  %s %s %s %s  %s"%(
+          prefix,
+          format_value("%s", pp.resname),
+          format_value("%7.3f", pp.max_dev),
+          format_value("%7.3f", pp.mean_dev),
+          pp.id)
+    if p.protein_planes_max_dev_noH is not None:
+      result += "\n\n%sMax deviation from planes (no H):"%prefix
+      result += "\n%s   Type  MaxDev  MeanDev LineInFile"%prefix
+      for pp in p.protein_planes_max_dev_noH:
+        result += "\n  %s %s %s %s  %s"%(
+          prefix,
+          format_value("%s", pp.resname),
+          format_value("%7.3f", pp.max_dev),
+          format_value("%7.3f", pp.mean_dev),
+          pp.id)
+    #
     if( uppercase ):
       result = result.upper()
     print(result, file=log)
@@ -446,8 +497,9 @@ class adp(object):
                pdb_hierarchy,
                xray_structure,
                use_hydrogens=False,
-               geometry_restraints_manager=None):
-    if(not use_hydrogens):
+               geometry_restraints_manager=None,
+               use_z_scores=False):
+    if(not use_hydrogens) or use_z_scores:
       not_hd_sel = ~xray_structure.hd_selection()
       pdb_hierarchy  = pdb_hierarchy.select(not_hd_sel)
       xray_structure = xray_structure.select(not_hd_sel)
@@ -455,6 +507,7 @@ class adp(object):
         geometry_restraints_manager = \
           geometry_restraints_manager.select(not_hd_sel)
     b_isos = xray_structure.extract_u_iso_or_u_equiv() * adptbx.u_as_b(1.)
+    if use_z_scores: mean = b_isos.as_z_scores()
     sites_cart = xray_structure.sites_cart()
     asc = pdb_hierarchy.atom_selection_cache()
     def get_stats(sel_str, rms_bonded=False):
@@ -631,3 +684,55 @@ class info(object):
     cif_block = self.model.tls_groups_as_cif_block(cif_block=cif_block)
     # What about anomalous_scatterer_groups here?
     return cif_block
+
+def protein_planes(hierarchy):
+  from cctbx import geometry_restraints
+  # XXX This needs to be generalized !!! XXX
+  planes = {
+  "ARG":     [' NE ', ' CZ ', ' NH1', ' NH2', ' HE ', 'HH11', 'HH12', 'HH21', 'HH22'],
+  "TYR":     ["CB","CG", "CD1","CD2","CE1","CE2","CZ", "OH", "HD1","HD2","HE1","HE2"],
+  "PHE":     ["CB","CG", "CD1","CD2","CE1","CE2","CZ", "OH", "HD1","HD2","HE1","HE2"],
+  "TRP":     ["CB","CG","CD1","CD2","NE1","CE2","CE3","CZ2","CZ3","CH2","HD1","HE1","HE3","HZ2","HZ3","HH2"],
+  "HIS":     [ "CG","ND1","CE1","NE2","CD2","HCD2","HNE2","HND1"]
+  }
+  asc = hierarchy.atom_selection_cache()
+  result = []
+  for resname in planes.keys():
+    MAX = -1
+    ID = None
+    RESNAME = None
+    deltas = flex.double()
+    names = planes[resname]
+    ss = ss = "resname %s and (%s)"%(resname," or ".join(["name %s"%it for it in names]))
+    sel = asc.selection(string=ss)
+    h = hierarchy.select(sel)
+    for residue_group in h.residue_groups():
+      if len(residue_group.atom_groups())>1: continue
+      atom_group = residue_group.atom_groups()[0]
+      assert atom_group.resname==resname
+      atoms = atom_group.atoms()
+      size = atoms.size()
+      sites_cart = atoms.extract_xyz()
+      pproxy = geometry_restraints.planarity_proxy(
+        i_seqs  = list(range(size)),
+        weights = flex.double([1]*size))
+      planarity = geometry_restraints.planarity(
+        sites_cart = sites_cart,
+        proxy      = pproxy)
+      abs_deltas = flex.abs(planarity.deltas())
+      deltas.extend(abs_deltas)
+      max_delta  = flex.max(abs_deltas)
+      if max_delta > MAX:
+        MAX = max_delta
+        ID = atom_group.id_str()
+        RESNAME = resname
+    if(MAX > 0 and deltas.size() > 0):
+      assert approx_equal(MAX, flex.max(deltas), 1.e-4)
+    r = group_args(
+      resname  = RESNAME,
+      max_dev  = MAX,
+      mean_dev = flex.mean(deltas) if deltas.size()>0 else None,
+      id       = ID)
+    if MAX>=0:
+      result.append(r)
+  return result
