@@ -51,6 +51,7 @@ general_tests = [
   "$D/regression/tst_ml_estimate.py",
   "$D/density_modification/tst_density_modification.py",
   "$D/geometry_restraints/tst_ramachandran.py",
+  "$D/geometry_restraints/tst_geo_file_parsing.py",
   "$D/geometry_restraints/tst_manager.py",
   "$D/geometry_restraints/external.py",
   "$D/regression/tst_map_type_parser.py",
@@ -79,6 +80,7 @@ general_tests = [
   "$D/regression/model/tst_model_ncs.py",
   "$D/regression/model/tst_model_2.py",
   "$D/regression/model/tst_model_biomt_mtrix.py",
+  "$D/regression/model/tst_model_biomt_mtrix_2.py",
   "$D/regression/model/tst_model_neutralize_scatterers.py",
   "$D/regression/model/tst_model_set_hydrogen_bond_length.py",
   "$D/regression/model/tst_model_cart_ref_restraints.py",
@@ -86,6 +88,7 @@ general_tests = [
   "$D/regression/model/tst_model_remove_alternative_conformations.py",
   "$D/regression/model/tst_model_get_vdw_radii.py",
   "$D/regression/model/tst_model_dedeuterate.py",
+  "$D/regression/model/tst_model_utils_merge_models.py",
   #
   "$D/regression/tst_fmodel.py",
   "$D/regression/tst_utils.py",
@@ -126,6 +129,7 @@ general_tests = [
   "$D/monomer_library/tst_selection.py",
   "$D/monomer_library/tst_tyr_from_gly_and_bnz.py",
   "$D/monomer_library/tst_pdb_interpretation.py",
+  "$D/monomer_library/tst_pdb_interpretation_3.py",
   "$D/monomer_library/tst_rna_dna_interpretation.py",
   "$D/monomer_library/tst_protein_interpretation.py",
   "$D/monomer_library/tst_pdb_interpretation_ncs_processing.py",
@@ -241,7 +245,6 @@ general_tests = [
   #
   "$D/idealized_aa_residues/tst.py",
   #
-  "$D/regression/tst_dssp.py",
   "$D/regression/tst_validation_summary.py",
   "$D/regression/tst_maps_misc.py",
   "$D/regression/tst_cablam.py",
@@ -290,9 +293,6 @@ general_tests = [
   "$D/building/tst_extend_sidechains.py",
   "$D/building/tst_extend_sidechains_2.py",
   # alt confs
-  "$D/building/alternate_conformations/tst_backrub_conformers.py",
-  "$D/building/alternate_conformations/tst_shear_conformers.py",
-  "$D/building/alternate_conformations/tst_partial_omit_map.py",
   "$D/disorder/tst.py",
   "$D/disorder/tst_backbone.py",
   "$D/disorder/tst_analyze_model.py",
@@ -380,7 +380,6 @@ general_tests = [
   "$D/ions/tst_environment.py",
   "$D/regression/tst_table_one.py",
   "$D/regression/tst_msa.py",
-  "$D/regression/tst_build_alt_confs.py",
   "$D/regression/tst_cc_star.py",
   "$D/regression/tst_fmodel_2.py",
   "$D/regression/tst_rigid_bond_test.py",
@@ -414,7 +413,8 @@ general_tests = [
   ]
 
 molprobity_tests = [
-  "$D/regression/tst_rama_z_01.py",
+  "$D/validation/regression/tst_rama_z_01.py",
+  "$D/validation/regression/tst_rama_z_02.py",
   "$D/regression/pdb_interpretation/tst_edits.py",
   "$D/regression/pdb_interpretation/tst_edits_actions.py",
   "$D/regression/tst_add_arrows_on_plot.py",
@@ -424,11 +424,12 @@ molprobity_tests = [
   "$D/regression/ncs/tst_geometry_minimization_ncs_constraints.py",
   "$D/regression/tst_clashes.py",
   "$D/validation/regression/tst_cbetadev.py",
+  "$D/validation/regression/tst_cbetadev_02.py",
   "$D/regression/tst_statistics_output.py",
   "$D/regression/tst_geo_min_restraints_phil.py",
   "$D/regression/tst_model_vs_map.py",
+  "$D/regression/tst_model_vs_sequence.py",
   # validation/molprobity
-  "$D/validation/regression/tst_nqh_minimize.py",
   "$D/validation/regression/tst_molprobity_1.py",
   "$D/validation/regression/tst_molprobity_4.py",
   "$D/validation/regression/tst_molprobity_5.py",
@@ -486,6 +487,11 @@ else:
   print("Skipping %d MolProbity tests: probe not configured"%len(molprobity_tests))
   tst_list = tuple(general_tests)
 
+# generally failing tests
+tst_list_expected_failures = [
+  "$D/monomer_library/tst_pdb_interpretation_2.py",
+  ]
+
 def run():
   assert easy_run.call("libtbx.find_untested mmtbx True")==0
   build_dir = libtbx.env.under_build("mmtbx")
