@@ -101,7 +101,8 @@ master_phil = iotbx.phil.parse("""
 
      ss_by_chain = None
        .type = bool
-       .help = Find secondary structure only within individual chains. \
+       .help = Only applies if search_method = from_ca. \
+              Find secondary structure only within individual chains. \
                Alternative is to allow H-bonds between chains. Can be \
                much slower with ss_by_chain=False. If your model is complete \
                use ss_by_chain=True. If your model is many fragments, use \
@@ -2131,7 +2132,7 @@ class replace_with_segments_from_pdb:
           first_residue_number=cg1_model.hierarchy.first_resseq_as_int(),
           renumber=True)
 
-        sites=combined_model.hierarchy.extract_xray_structure().sites_cart()
+        sites=combined_model.hierarchy.atoms().extract_xyz()
 
         # now select a few possibilities with at least 1 res overlap on each end
         #  and no more than other.standard_length residues
