@@ -198,6 +198,12 @@ def _rms(data):
   if den <= 0: return None
   return (sum(pow2(data)) / den)**0.5
 
+def _as_z_scores(self):
+  rc=min_max_mean_double(values=self)
+  ssd = _sample_standard_deviation(self)
+  self -= rc.mean
+  self /= ssd
+
 double.format_min = _format_min
 double.format_max = _format_max
 double.format_mean = _format_mean
@@ -205,6 +211,7 @@ double.min_max_mean = _min_max_mean_double_init
 double.standard_deviation_of_the_sample = _standard_deviation_of_the_sample
 double.sample_standard_deviation = _sample_standard_deviation
 double.rms = _rms
+double.as_z_scores = _as_z_scores
 
 def select(sequence, permutation=None, flags=None):
   result = []
