@@ -5,7 +5,7 @@ from __future__ import print_function, division
 import numpy as np
 import sys
 import time
-from simtbx.command_line.hopper import single_expt_pandas
+from simtbx.diffBragg.hopper_io import single_expt_pandas
 from copy import deepcopy
 import os
 from libtbx.mpi4py import  MPI
@@ -25,6 +25,7 @@ from pylab import plt
 from scipy.optimize import basinhopping
 if COMM.rank > 0:
     sys.tracebacklimit = 0
+from simtbx.diffBragg import psf
 
 # diffBragg internal indices for derivative manager
 ROTXYZ_ID = hopper_utils.ROTXYZ_IDS
@@ -40,6 +41,34 @@ PAN_XYZ_IDS = PAN_X_ID, PAN_Y_ID, PAN_Z_ID
 DEG_TO_PI = np.pi/180.
 
 
+def convolve_model_with_psf(model_pix, SIM, pan_fast_slow, roi_id_slices, roi_id_unique):
+
+    if not SIM.use_psf:
+        return model_pix
+    PSF = SIM.PSF
+    psf_args = SIM.psf_args
+
+    coords = pan_fast_slow.as_numpy_array()
+    fid = coords[1::3]
+    sid = coords[2::3]
+
+    for i in roi_id_unique:
+        for slc in roi_id_slices[i]:
+            fvals = fid[slc]
+            svals = sid[slc]
+            f0 = fvals.min()
+            s0 = svals.min()
+            f1 = fvals.max()
+            s1 = svals.max()
+            fdim = int(f1-f0+1)
+            sdim = int(s1-s0+1)
+            img = model_pix[slc].reshape((sdim, fdim))
+            img = psf.convolve_with_psf(img, psf=PSF, **psf_args)
+            model_pix[slc] = img.ravel()
+
+    return model_pix
+
+
 def get_dist_from_R(R):
     """ returns prediction offset, R is reflection table"""
     x, y, _ = R['xyzobs.px.value'].parts()
@@ -48,9 +77,43 @@ def get_dist_from_R(R):
     return dist
 
 
+class BeamParameters:
+    def __init__(self, phil_params, data_modelers):
+        self.parameters = []
+        # initialize as the median of all lam0, lam1 values
+        all_lam0 = []
+        all_lam1 = []
+        for i_m in data_modelers:
+            m = data_modelers[i_m]
+            spec0, spec1 = m.PAR.spec_coef
+            lam0, lam1 = spec0.init, spec1.init
+            all_lam0.append(lam0)
+            all_lam1.append(lam1)
+        all_lam0 = COMM.reduce(all_lam0)
+        all_lam1 = COMM.reduce(all_lam1)
+        global_lam0 = global_lam1 = None
+        if COMM.rank==0:
+            global_lam0 = np.median(all_lam0)
+            global_lam1 = np.median(all_lam1)
+        global_lam0 = COMM.bcast(global_lam0)
+        global_lam1 = COMM.bcast(global_lam1)
+
+        for i_p, init_val in enumerate((global_lam0, global_lam1)):
+            p = RangedParameter(name="lambda%d" % i_p,
+                init=init_val,
+                sigma=phil_params.sigmas.spec[i_p],
+                minval=phil_params.mins.spec[i_p],
+                maxval=phil_params.maxs.spec[i_p],
+                fix=phil_params.fix.spec,
+                center=phil_params.centers.spec[i_p],
+                beta=phil_params.betas.spec[i_p],
+                is_global=True)
+            self.parameters.append(p)
+
+
 class DetectorParameters:
 
-    def __init__(self, phil_params, panel_groups_refined, num_panel_groups ):
+    def __init__(self, phil_params, panel_groups_refined, num_panel_groups):
 
         self.parameters = []
         GEO = phil_params.geometry
@@ -63,14 +126,14 @@ class DetectorParameters:
 
             o = RangedParameter(name="group%d_RotOrth" % i_group,
                                 init=0,
-                                sigma=1,  # TODO
+                                sigma=100,  # TODO
                                 minval=GEO.min.panel_rotations[0]*DEG_TO_PI,
                                 maxval=GEO.max.panel_rotations[0]*DEG_TO_PI,
                                 fix=not vary_rots[0], center=0, beta=GEO.betas.panel_rot[0], is_global=True)
 
             f = RangedParameter(name="group%d_RotFast" % i_group,
                                 init=0,
-                                sigma=1,  # TODO
+                                sigma=100,  # TODO
                                 minval=GEO.min.panel_rotations[1]*DEG_TO_PI,
                                 maxval=GEO.max.panel_rotations[1]*DEG_TO_PI,
                                 fix=not vary_rots[1], center=0, beta=GEO.betas.panel_rot[1],
@@ -78,7 +141,7 @@ class DetectorParameters:
 
             s = RangedParameter(name="group%d_RotSlow" % i_group,
                                 init=0,
-                                sigma=1,  # TODO
+                                sigma=100,  # TODO
                                 minval=GEO.min.panel_rotations[2]*DEG_TO_PI,
                                 maxval=GEO.max.panel_rotations[2]*DEG_TO_PI,
                                 fix=not vary_rots[2], center=0, beta=GEO.betas.panel_rot[2],
@@ -87,17 +150,17 @@ class DetectorParameters:
             vary_shifts = [not fixed_flag and group_has_data for fixed_flag in GEO.fix.panel_translations]
             #vary_shifts = [True]*3
             x = RangedParameter(name="group%d_ShiftX" % i_group, init=0,
-                                sigma=1,
+                                sigma=100,
                                 minval=GEO.min.panel_translations[0]*1e-3, maxval=GEO.max.panel_translations[0]*1e-3,
                                 fix=not vary_shifts[0], center=0, beta=GEO.betas.panel_xyz[0],
                                 is_global=True)
             y = RangedParameter(name="group%d_ShiftY" % i_group, init=0,
-                                sigma=1,
+                                sigma=100,
                                 minval=GEO.min.panel_translations[1]*1e-3, maxval=GEO.max.panel_translations[1]*1e-3,
                                 fix=not vary_shifts[1], center=0, beta=GEO.betas.panel_xyz[1],
                                 is_global=True)
             z = RangedParameter(name="group%d_ShiftZ" % i_group, init=0,
-                                sigma=1,
+                                sigma=100,
                                 minval=GEO.min.panel_translations[2]*1e-3, maxval=GEO.max.panel_translations[2]*1e-3,
                                 fix=not vary_shifts[2], center=0, beta=GEO.betas.panel_xyz[2],
                                 is_global=True)
@@ -105,6 +168,7 @@ class DetectorParameters:
             self.parameters += [o, f, s, x, y, z]
 
 
+
 class CrystalParameters:
 
     def __init__(self, phil_params, data_modelers):
@@ -138,6 +202,20 @@ class CrystalParameters:
                                           center=p.center, beta=p.beta)
                 self.parameters.append(ref_p)
 
+            for i_N in range(3):
+                p = Mod.PAR.Ndef[i_N]
+                ref_p = RangedParameter(name="rank%d_shot%d_Ndef%d" % (COMM.rank, i_shot, i_N),
+                                        minval=p.minval, maxval=p.maxval, fix=self.phil.fix.Ndef, init=p.init,
+                                        center=p.center, beta=p.beta)
+                self.parameters.append(ref_p)
+
+            for i_eta in range(3):
+                p = Mod.PAR.eta[i_eta]
+                ref_p = RangedParameter(name="rank%d_shot%d_eta%d" % (COMM.rank, i_shot, i_eta),
+                                        minval=p.minval, maxval=p.maxval, fix=self.phil.fix.eta_abc, init=p.init,
+                                        center=p.center, beta=p.beta)
+                self.parameters.append(ref_p)
+
             for i_rot in range(3):
                 p = Mod.PAR.RotXYZ_params[i_rot]
                 ref_p = RangedParameter(name="rank%d_shot%d_RotXYZ%d" % (COMM.rank, i_shot, i_rot),
@@ -228,7 +306,7 @@ class Target:
             print("Final Iteration %d:\n\tResid=%f, sigmaZ %f" % (self.iternum, f, self.sigmaZ))
 
 
-def model(x, ref_params, i_shot, Modeler, SIM, return_model=False):
+def model(x, ref_params, i_shot, Modeler, SIM, return_bragg_model=False):
     """
 
     :param x: rescaled parameter array (global)
@@ -238,7 +316,7 @@ def model(x, ref_params, i_shot, Modeler, SIM, return_model=False):
         have names which include i_shot
     :param Modeler: DataModeler for i_shot
     :param SIM: instance of sim_data.SimData
-    :param return_model: if true, bypass the latter half of the method and return the Bragg scattering model
+    :param return_bragg_model: if true, bypass the latter half of the method and return the Bragg scattering model
     :return: either the Bragg scattering model (if return_model), or else a 3-tuple of
         (float, dict of float, float)
         (negative log likelihood, gradient of negative log likelihood, average sigmaZ for the shot)
@@ -250,9 +328,59 @@ def model(x, ref_params, i_shot, Modeler, SIM, return_model=False):
     Na = ref_params["rank%d_shot%d_Nabc%d" % (COMM.rank, i_shot, 0)]
     Nb = ref_params["rank%d_shot%d_Nabc%d" % (COMM.rank, i_shot, 1)]
     Nc = ref_params["rank%d_shot%d_Nabc%d" % (COMM.rank, i_shot, 2)]
+    Nd = ref_params["rank%d_shot%d_Ndef%d" % (COMM.rank, i_shot, 0)]
+    Ne = ref_params["rank%d_shot%d_Ndef%d" % (COMM.rank, i_shot, 1)]
+    Nf = ref_params["rank%d_shot%d_Ndef%d" % (COMM.rank, i_shot, 2)]
+    eta_a = ref_params["rank%d_shot%d_eta%d" % (COMM.rank, i_shot, 0)]
+    eta_b = ref_params["rank%d_shot%d_eta%d" % (COMM.rank, i_shot, 1)]
+    eta_c = ref_params["rank%d_shot%d_eta%d" % (COMM.rank, i_shot, 2)]
     G = ref_params["rank%d_shot%d_Scale" % (COMM.rank, i_shot)]
     num_uc_p = len(Modeler.ucell_man.variables)
     ucell_pars = [ref_params["rank%d_shot%d_Ucell%d" % (COMM.rank, i_shot, i_uc)] for i_uc in range(num_uc_p)]
+    lam0 = ref_params["lambda0"]
+    lam1 = ref_params["lambda1"]
+
+    # update the rotational mosaicity here
+    # update the mosaicity here
+    eta_params = [eta_a, eta_b, eta_c]
+    if SIM.umat_maker is not None:
+        # we are modeling mosaic spread
+        eta_abc = [p.get_val(x[p.xpos]) for p in eta_params]
+        #if not SIM.D.has_anisotropic_mosaic_spread:
+        #    eta_abc = eta_abc[0]
+        SIM.update_umats_for_refinement(eta_abc)
+
+    # update the photon energy spectrum for this shot
+    SIM.beam.spectrum = Modeler.spectra
+    SIM.D.xray_beams = SIM.beam.xray_beams
+    # update the lambda coeff
+    lambda_coef = lam0.get_val(x[lam0.xpos]), lam1.get_val(x[lam1.xpos])
+    SIM.D.lambda_coefficients = lambda_coef
+
+    # update the Bmatrix
+    Modeler.ucell_man.variables = [p.get_val(x[p.xpos]) for p in ucell_pars]
+    Bmatrix = Modeler.ucell_man.B_recipspace
+    SIM.D.Bmatrix = Bmatrix
+    for i_ucell in range(len(ucell_pars)):
+        SIM.D.set_ucell_derivative_matrix(
+            i_ucell + hopper_utils.UCELL_ID_OFFSET,
+            Modeler.ucell_man.derivative_matrices[i_ucell])
+    eta_a = ref_params["rank%d_shot%d_eta%d" % (COMM.rank, i_shot, 0)]
+    eta_b = ref_params["rank%d_shot%d_eta%d" % (COMM.rank, i_shot, 1)]
+    eta_c = ref_params["rank%d_shot%d_eta%d" % (COMM.rank, i_shot, 2)]
+    G = ref_params["rank%d_shot%d_Scale" % (COMM.rank, i_shot)]
+    num_uc_p = len(Modeler.ucell_man.variables)
+    ucell_pars = [ref_params["rank%d_shot%d_Ucell%d" % (COMM.rank, i_shot, i_uc)] for i_uc in range(num_uc_p)]
+
+    # update the rotational mosaicity here
+    # update the mosaicity here
+    eta_params = [eta_a, eta_b, eta_c]
+    if SIM.umat_maker is not None:
+        # we are modeling mosaic spread
+        eta_abc = [p.get_val(x[p.xpos]) for p in eta_params]
+        #if not SIM.D.has_anisotropic_mosaic_spread:
+        #    eta_abc = eta_abc[0]
+        SIM.update_umats_for_refinement(eta_abc)
 
     # update the photon energy spectrum for this shot
     SIM.beam.spectrum = Modeler.spectra
@@ -277,6 +405,9 @@ def model(x, ref_params, i_shot, Modeler, SIM, return_model=False):
     SIM.D.set_ncells_values((Na.get_val(x[Na.xpos]),
                              Nb.get_val(x[Nb.xpos]),
                              Nc.get_val(x[Nc.xpos])))
+    SIM.D.Ncells_def = (Nd.get_val(x[Nd.xpos]),
+                       Ne.get_val(x[Ne.xpos]),
+                       Nf.get_val(x[Nf.xpos]))
 
     npix = int(len(Modeler.pan_fast_slow)/3.)
 
@@ -304,12 +435,14 @@ def model(x, ref_params, i_shot, Modeler, SIM, return_model=False):
 
     # scale the bragg scattering
     bragg = all_bragg_scales*bragg_no_scale
+    if return_bragg_model:
+        return bragg
 
     # this is the total forward model:
     model_pix = bragg + Modeler.all_background
+    if SIM.use_psf:
+        model_pix = convolve_model_with_psf(model_pix, SIM,  Modeler.pan_fast_slow, roi_id_slices=Modeler.roi_id_slices, roi_id_unique=Modeler.roi_id_unique)
 
-    if return_model:
-        return model_pix
 
     # compute the negative log Likelihood
     resid = (Modeler.all_data - model_pix)
@@ -333,14 +466,24 @@ def model(x, ref_params, i_shot, Modeler, SIM, return_model=False):
             scale_fac, p = perRoiScaleFactors[roi_id]
             slc = Modeler.roi_id_slices[roi_id][0]  # theres just one slice for each roi_id
             d = p.get_deriv(x[p.xpos], bragg_no_roi[slc])
+
+            if SIM.use_psf:
+                x1,x2,y1,y2 = Modeler.rois[roi_id]
+                sdim, fdim = y2-y1, x2-x1
+                d_img = d.reshape((sdim, fdim))
+                d_img = psf.convolve_with_psf(d_img, psf=SIM.PSF, **SIM.psf_args)
+                d = d_img.ravel()
+
             d_trusted = Modeler.all_trusted[slc]
             common_term_slc = common_grad_term[slc]
             J[p.name] = (common_term_slc*d)[d_trusted].sum()
 
     # scale factor gradients
+    conv_args = {"SIM": SIM, "pan_fast_slow": Modeler.pan_fast_slow, "roi_id_slices": Modeler.roi_id_slices, "roi_id_unique": Modeler.roi_id_unique}
     if not G.fix:
         bragg_no_roi_scale = bragg_no_scale*Modeler.per_roi_scales_per_pix
         scale_grad = G.get_deriv(x[G.xpos], bragg_no_roi_scale)
+        scale_grad = convolve_model_with_psf(scale_grad, **conv_args)
         J[G.name] = (common_grad_term*scale_grad)[Modeler.all_trusted].sum()
 
     # Umat gradients
@@ -349,6 +492,7 @@ def model(x, ref_params, i_shot, Modeler, SIM, return_model=False):
             rot_db_id = ROTXYZ_ID[i_rot]
             rot_grad = scale*SIM.D.get_derivative_pixels(rot_db_id).as_numpy_array()[:npix]
             rot_grad = rot.get_deriv(x[rot.xpos], rot_grad)
+            rot_grad = convolve_model_with_psf(rot_grad, **conv_args)
             J[rot.name] = (common_grad_term*rot_grad)[Modeler.all_trusted].sum()
 
     # mosaic block size gradients
@@ -357,20 +501,54 @@ def model(x, ref_params, i_shot, Modeler, SIM, return_model=False):
         for i_N, N in enumerate([Na, Nb, Nc]):
             N_grad = scale*(Nabc_grad[i_N][:npix].as_numpy_array())
             N_grad = N.get_deriv(x[N.xpos], N_grad)
+            N_grad = convolve_model_with_psf(N_grad, **conv_args)
             J[N.name] = (common_grad_term*N_grad)[Modeler.all_trusted].sum()
 
+    if not Nd.fix:
+        Ndef_grad = SIM.D.get_ncells_def_derivative_pixels()
+        for i_N, N in enumerate([Nf, Ne, Nf]):
+            N_grad = scale*(Ndef_grad[i_N][:npix].as_numpy_array())
+            N_grad = N.get_deriv(x[N.xpos], N_grad)
+            N_grad = convolve_model_with_psf(N_grad, **conv_args)
+            J[N.name] = (common_grad_term*N_grad)[Modeler.all_trusted].sum()
+
+    if not eta_a.fix:
+        if SIM.D.has_anisotropic_mosaic_spread:
+            eta_abc_derivs = SIM.D.get_aniso_eta_deriv_pixels()
+        else:
+            eta_abc_derivs = [SIM.D.get_derivative_pixels(hopper_utils.ETA_ID)]
+        for i_eta, eta in enumerate(eta_params):
+            eta_grad = scale*(eta_abc_derivs[i_eta][:npix].as_numpy_array())
+            eta_grad = eta.get_deriv(x[eta.xpos], eta_grad)
+            eta_grad = convolve_model_with_psf(eta_grad, **conv_args)
+            J[eta.name] = (common_grad_term*eta_grad)[Modeler.all_trusted].sum()
+            if not SIM.D.has_anisotropic_mosaic_spread:
+                break
+
     # unit cell gradients
     if not ucell_pars[0].fix:
         for i_ucell, uc_p in enumerate(ucell_pars):
             d = scale*SIM.D.get_derivative_pixels(hopper_utils.UCELL_ID_OFFSET+i_ucell).as_numpy_array()[:npix]
             d = uc_p.get_deriv(x[uc_p.xpos], d)
+            d = convolve_model_with_psf(d, **conv_args)
             J[ucell_pars[i_ucell].name] = (common_grad_term*d)[Modeler.all_trusted].sum()
 
+    if not lam0.fix:
+        lambda_derivs = SIM.D.get_lambda_derivative_pixels()
+        lam_params = lam0, lam1
+        for d, pr in zip(lambda_derivs, lam_params):
+            d = d.as_numpy_array()[:npix]
+            d = pr.get_deriv(x[pr.xpos], d)
+            d = convolve_model_with_psf(d, **conv_args)
+            J[pr.name] = (common_grad_term*d)[Modeler.all_trusted].sum()
+
     # detector model gradients
     detector_derivs = []
     for diffbragg_parameter_id in PAN_OFS_IDS+PAN_XYZ_IDS:
         try:
-            d = common_grad_term*scale*(SIM.D.get_derivative_pixels(diffbragg_parameter_id).as_numpy_array()[:npix])
+            d = SIM.D.get_derivative_pixels(diffbragg_parameter_id).as_numpy_array()[:npix]
+            d = convolve_model_with_psf(d, **conv_args)
+            d = common_grad_term*scale*d
         except ValueError:
             d = None
         detector_derivs.append(d)
@@ -472,8 +650,7 @@ def update_detector(x, ref_params, SIM, save=None):
         El.append(E)
         El.as_file(save)
         t = time.time()-t
-        print("Saved detector model to %s (took %.4f sec)" % (save, t), flush=True )
-
+        #print("Saved detector model to %s (took %.4f sec)" % (save, t), flush=True )
 
 
 def target_and_grad(x, ref_params, data_modelers, SIM, params):
@@ -504,6 +681,8 @@ def target_and_grad(x, ref_params, data_modelers, SIM, params):
 
         if params.use_restraints:
             for name in ref_params:
+                if name.startswith("Fhkl"):
+                    continue
                 par = ref_params[name]
                 if not par.is_global and not par.fix:
                     val = par.get_restraint_val(x[par.xpos])
@@ -528,6 +707,8 @@ def target_and_grad(x, ref_params, data_modelers, SIM, params):
     ## add in the detector parameter restraints
     if params.use_restraints:
         for name in ref_params:
+            if name.startswith("Fhkl"):
+                continue
             par = ref_params[name]
             if par.is_global and not par.fix:
                 target_functional += par.get_restraint_val(x[par.xpos])
@@ -579,6 +760,10 @@ def geom_min(params):
             os.makedirs(pdir)
     if COMM.rank == 0:
         print("Will optimize using %d experiments" %len(df))
+
+    from simtbx.diffBragg import mpi_logger
+    mpi_logger.setup_logging_from_params(params)
+
     launcher.load_inputs(df, refls_key=params.geometry.refls_key)
 
     for i_shot in launcher.Modelers:
@@ -588,14 +773,20 @@ def geom_min(params):
     # same on every rank:
     det_params = DetectorParameters(params, launcher.panel_groups_refined, launcher.n_panel_groups)
 
+    beam_params = BeamParameters(params, launcher.Modelers)
+
     # different on each rank
     crystal_params = CrystalParameters(params,launcher.Modelers)
     crystal_params.parameters = COMM.bcast(COMM.reduce(crystal_params.parameters))
 
     LMP = Parameters()
-    for p in crystal_params.parameters + det_params.parameters:
+    for p in crystal_params.parameters + det_params.parameters + beam_params.parameters:
         LMP.add(p)
 
+    # use spectrum coefficients
+    launcher.SIM.D.use_lambda_coefficients = True
+    launcher.SIM.D.lambda_coefficients = LMP["lambda0"].init, LMP["lambda1"].init
+
     # attached some objects to SIM for convenience
     launcher.SIM.panel_reference_from_id = launcher.panel_reference_from_id
     launcher.SIM.panel_group_from_id = launcher.panel_group_from_id
@@ -614,8 +805,15 @@ def geom_min(params):
     if not params.fix.RotXYZ:
         for i_rot in range(3):
             launcher.SIM.D.refine(ROTXYZ_ID[i_rot])
+    if not params.fix.spec:
+        launcher.SIM.D.refine(hopper_utils.LAMBDA_IDS[0])
+        launcher.SIM.D.refine(hopper_utils.LAMBDA_IDS[1])
+    if not params.fix.eta_abc:
+        launcher.SIM.D.refine(hopper_utils.ETA_ID)
     if not params.fix.Nabc:
         launcher.SIM.D.refine(hopper_utils.NCELLS_ID)
+    if not params.fix.Ndef:
+        launcher.SIM.D.refine(hopper_utils.NCELLS_ID_OFFDIAG)
     if not params.fix.ucell:
         for i_ucell in range(launcher.SIM.num_ucell_param):
             launcher.SIM.D.refine(hopper_utils.UCELL_ID_OFFSET + i_ucell)
@@ -674,6 +872,12 @@ def write_output_files(Xopt, LMP, Modelers, SIM, params):
             if not os.path.exists(dname):
                 os.makedirs(dname)
 
+    lam0_lam1 = []
+    for i_lam in [0,1]:
+        lam_p = LMP["lambda%d"% i_lam]
+        val = lam_p.get_val(Xopt[lam_p.xpos])
+        lam0_lam1.append(val)
+
     all_shot_pred_offsets = []
     for i_shot in Modelers:
         Modeler = Modelers[i_shot]
@@ -703,8 +907,8 @@ def write_output_files(Xopt, LMP, Modelers, SIM, params):
         new_exp.beam.set_wavelength(ave_wave)
         new_exp.detector = opt_det
 
-        Modeler.best_model = model(Xopt, LMP, i_shot, Modeler, SIM, return_model=True)
-        Modeler.best_model_includes_background = True
+        Modeler.best_model = model(Xopt, LMP, i_shot, Modeler, SIM, return_bragg_model=True)
+        Modeler.best_model_includes_background = False
 
         # store the updated per-roi scale factors in the new refl table
         roi_scale_factor = flex.double(len(Modeler.refls), 1)
@@ -715,7 +919,7 @@ def write_output_files(Xopt, LMP, Modelers, SIM, params):
             slc = Modeler.roi_id_slices[roi_id][0]
             roi_refl_ids = Modeler.all_refls_idx[slc]
             # NOTE, just a sanity check:
-            assert len(np.unique(roi_refl_ids))==1
+            assert len(np.unique(roi_refl_ids))==1, "unique refl ids"
             refl_idx = roi_refl_ids[0]
             assert test_refl_idx==refl_idx
             roi_scale_factor[refl_idx] = scale_fac
@@ -747,15 +951,21 @@ def write_output_files(Xopt, LMP, Modelers, SIM, params):
         if params.geometry.pandas_dir is not None:
             a,b,c,al,be,ga = ucpar
             ncells_p = [LMP["rank%d_shot%d_Nabc%d" % (COMM.rank, i_shot, i)] for i in range(3)]
+            ncells_def_p = [LMP["rank%d_shot%d_Ndef%d" % (COMM.rank, i_shot, i)] for i in range(3)]
             Na,Nb,Nc = [p.get_val(Xopt[p.xpos]) for p in ncells_p]
+            Nd,Ne,Nf = [p.get_val(Xopt[p.xpos]) for p in ncells_def_p]
+
+            eta_p = [LMP["rank%d_shot%d_eta%d" % (COMM.rank, i_shot, i)] for i in range(3)]
+            eta_abc = tuple([p.get_val(Xopt[p.xpos]) for p in eta_p])
+
             scale_p = LMP["rank%d_shot%d_Scale" %(COMM.rank, i_shot)]
             scale = scale_p.get_val(Xopt[scale_p.xpos])
 
             _,fluxes = zip(*SIM.beam.spectrum)
-            eta_a = eta_b = eta_c = np.nan
+            # TODO OUTPUTDEF and LAM0, LAM1
             df= single_expt_pandas(xtal_scale=scale, Amat=new_crystal.get_A(),
-                ncells_abc=(Na, Nb, Nc), ncells_def=(0,0,0),
-                eta_abc=(eta_a, eta_b, eta_c),
+                ncells_abc=(Na, Nb, Nc), ncells_def=(Nd, Ne, Nf),
+                eta_abc=eta_abc,
                 diff_gamma=(np.nan, np.nan, np.nan),
                 diff_sigma=(np.nan, np.nan, np.nan),
                 detz_shift=0,
@@ -765,7 +975,7 @@ def write_output_files(Xopt, LMP, Modelers, SIM, params):
                 rotXYZ=tuple(rotXYZ),
                 ucell_p = (a,b,c,al,be,ga),
                 ucell_p_init=(np.nan, np.nan, np.nan, np.nan, np.nan, np.nan),
-                lam0_lam1 = (np.nan, np.nan),
+                lam0_lam1 = lam0_lam1,
                 spec_file=Modeler.spec_name,
                 spec_stride=params.simulator.spectrum.stride,
                 flux=sum(fluxes), beamsize_mm=SIM.beam.size_mm,
