@@ -7,6 +7,9 @@ using Kokkos::finalize;
 namespace simtbx {
 namespace Kokkos {
 
+  bool kokkos_instance::m_isInitialized = false;
+  int kokkos_instance::m_instances = 0;
+
   kokkos_instance::kokkos_instance() {
     printf("NO OPERATION, NO DEVICE NUMBER");
   }
@@ -15,24 +18,30 @@ namespace Kokkos {
     InitArguments kokkos_init;
     kokkos_init.device_id = t_deviceID;
 
-    initialize(kokkos_init);
-    bFinalized = false;
-    deviceID = t_deviceID;
+    if (!m_isInitialized) {
+      initialize(kokkos_init);
+
+      m_isInitialized = true;
+      m_isFinalized = false;
+      m_deviceID = t_deviceID;
+    }
+    ++m_instances;
   }
 
   int
   kokkos_instance::get_deviceID() const {
-    return deviceID;
+    return m_deviceID;
   }
 
   void
   kokkos_instance::finalize_kokkos() {
     finalize();
-    bFinalized = true;
+    m_isFinalized = true;
   }
 
   kokkos_instance::~kokkos_instance() {
-    if (!bFinalized) { finalize(); }
+    --m_instances;
+    if (!m_isFinalized && m_instances<1) { finalize(); }
   }
 
 
