@@ -443,11 +443,19 @@ class ProcessDialog(wx.Dialog):
   def OnAbort(self, event):
     self.process.abort()
     self._aborted = True
-    self.EndModal(wx.ID_CANCEL)
+    self._timer.Stop()
+    try:
+      self.EndModal(wx.ID_CANCEL)
+    except Exception as e:
+      pass # C++ was deleted
 
   def OnError(self, event):
     self._error = event.data
-    self.EndModal(wx.ID_CANCEL)
+    self._timer.Stop()
+    try:
+      self.EndModal(wx.ID_CANCEL)
+    except Exception as e:
+      pass # C++ was deleted
 
   def exception_raised(self):
     return (self._error is not None)
